"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class StopSparkMsg(google.protobuf.message.Message):
    """Message that the client sends to stop spark"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StopSparkMsg = StopSparkMsg

@typing_extensions.final
class StartSparkMsg(google.protobuf.message.Message):
    """Message that the client sends to start spark"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StartSparkMsg = StartSparkMsg

@typing_extensions.final
class StopHostMonitorMsg(google.protobuf.message.Message):
    """Message that the client sends to stop the HostMonitor"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StopHostMonitorMsg = StopHostMonitorMsg

@typing_extensions.final
class StartHostMonitorMsg(google.protobuf.message.Message):
    """Message that the client sends to start the HostMonitor"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KAFKA_IP_FIELD_NUMBER: builtins.int
    KAFKA_PORT_FIELD_NUMBER: builtins.int
    TIME_STEP_LEN_SECONDS_FIELD_NUMBER: builtins.int
    kafka_ip: builtins.str
    kafka_port: builtins.int
    time_step_len_seconds: builtins.int
    def __init__(
        self,
        *,
        kafka_ip: builtins.str = ...,
        kafka_port: builtins.int = ...,
        time_step_len_seconds: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["kafka_ip", b"kafka_ip", "kafka_port", b"kafka_port", "time_step_len_seconds", b"time_step_len_seconds"]) -> None: ...

global___StartHostMonitorMsg = StartHostMonitorMsg

@typing_extensions.final
class StopFilebeatMsg(google.protobuf.message.Message):
    """Message that the client sends to stop filebeat"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StopFilebeatMsg = StopFilebeatMsg

@typing_extensions.final
class StartFilebeatMsg(google.protobuf.message.Message):
    """Message that the client sends to start filebeat"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StartFilebeatMsg = StartFilebeatMsg

@typing_extensions.final
class ConfigFilebeatMsg(google.protobuf.message.Message):
    """Message that the client sends to set the filebeat configuration"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KIBANA_IP_FIELD_NUMBER: builtins.int
    KIBANA_PORT_FIELD_NUMBER: builtins.int
    ELASTIC_IP_FIELD_NUMBER: builtins.int
    ELASTIC_PORT_FIELD_NUMBER: builtins.int
    NUM_ELASTIC_SHARDS_FIELD_NUMBER: builtins.int
    RELOAD_ENABLED_FIELD_NUMBER: builtins.int
    KAFKA_IP_FIELD_NUMBER: builtins.int
    KAFKA_PORT_FIELD_NUMBER: builtins.int
    KAFKA_FIELD_NUMBER: builtins.int
    KAFKA_TOPICS_FIELD_NUMBER: builtins.int
    FILEBEAT_MODULES_FIELD_NUMBER: builtins.int
    LOG_FILES_PATHS_FIELD_NUMBER: builtins.int
    kibana_ip: builtins.str
    kibana_port: builtins.int
    elastic_ip: builtins.str
    elastic_port: builtins.int
    num_elastic_shards: builtins.int
    reload_enabled: builtins.bool
    kafka_ip: builtins.str
    kafka_port: builtins.int
    kafka: builtins.bool
    @property
    def kafka_topics(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    @property
    def filebeat_modules(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    @property
    def log_files_paths(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        kibana_ip: builtins.str = ...,
        kibana_port: builtins.int = ...,
        elastic_ip: builtins.str = ...,
        elastic_port: builtins.int = ...,
        num_elastic_shards: builtins.int = ...,
        reload_enabled: builtins.bool = ...,
        kafka_ip: builtins.str = ...,
        kafka_port: builtins.int = ...,
        kafka: builtins.bool = ...,
        kafka_topics: collections.abc.Iterable[builtins.str] | None = ...,
        filebeat_modules: collections.abc.Iterable[builtins.str] | None = ...,
        log_files_paths: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["elastic_ip", b"elastic_ip", "elastic_port", b"elastic_port", "filebeat_modules", b"filebeat_modules", "kafka", b"kafka", "kafka_ip", b"kafka_ip", "kafka_port", b"kafka_port", "kafka_topics", b"kafka_topics", "kibana_ip", b"kibana_ip", "kibana_port", b"kibana_port", "log_files_paths", b"log_files_paths", "num_elastic_shards", b"num_elastic_shards", "reload_enabled", b"reload_enabled"]) -> None: ...

global___ConfigFilebeatMsg = ConfigFilebeatMsg

@typing_extensions.final
class StopPacketbeatMsg(google.protobuf.message.Message):
    """Message that the client sends to stop packetbeat"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StopPacketbeatMsg = StopPacketbeatMsg

@typing_extensions.final
class StartPacketbeatMsg(google.protobuf.message.Message):
    """Message that the client sends to start packetbeat"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StartPacketbeatMsg = StartPacketbeatMsg

@typing_extensions.final
class ConfigPacketbeatMsg(google.protobuf.message.Message):
    """Message that the client sends to set the packetbeat configuration"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KIBANA_IP_FIELD_NUMBER: builtins.int
    KIBANA_PORT_FIELD_NUMBER: builtins.int
    ELASTIC_IP_FIELD_NUMBER: builtins.int
    ELASTIC_PORT_FIELD_NUMBER: builtins.int
    NUM_ELASTIC_SHARDS_FIELD_NUMBER: builtins.int
    kibana_ip: builtins.str
    kibana_port: builtins.int
    elastic_ip: builtins.str
    elastic_port: builtins.int
    num_elastic_shards: builtins.int
    def __init__(
        self,
        *,
        kibana_ip: builtins.str = ...,
        kibana_port: builtins.int = ...,
        elastic_ip: builtins.str = ...,
        elastic_port: builtins.int = ...,
        num_elastic_shards: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["elastic_ip", b"elastic_ip", "elastic_port", b"elastic_port", "kibana_ip", b"kibana_ip", "kibana_port", b"kibana_port", "num_elastic_shards", b"num_elastic_shards"]) -> None: ...

global___ConfigPacketbeatMsg = ConfigPacketbeatMsg

@typing_extensions.final
class StopMetricbeatMsg(google.protobuf.message.Message):
    """Message that the client sends to stop metricbeat"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StopMetricbeatMsg = StopMetricbeatMsg

@typing_extensions.final
class StartMetricbeatMsg(google.protobuf.message.Message):
    """Message that the client sends to start metricbeat"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StartMetricbeatMsg = StartMetricbeatMsg

@typing_extensions.final
class ConfigMetricbeatMsg(google.protobuf.message.Message):
    """Message that the client sends to set the metricbeat configuration"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KIBANA_IP_FIELD_NUMBER: builtins.int
    KIBANA_PORT_FIELD_NUMBER: builtins.int
    ELASTIC_IP_FIELD_NUMBER: builtins.int
    ELASTIC_PORT_FIELD_NUMBER: builtins.int
    NUM_ELASTIC_SHARDS_FIELD_NUMBER: builtins.int
    KAFKA_IP_FIELD_NUMBER: builtins.int
    KAFKA_PORT_FIELD_NUMBER: builtins.int
    RELOAD_ENABLED_FIELD_NUMBER: builtins.int
    METRICBEAT_MODULES_FIELD_NUMBER: builtins.int
    kibana_ip: builtins.str
    kibana_port: builtins.int
    elastic_ip: builtins.str
    elastic_port: builtins.int
    num_elastic_shards: builtins.int
    kafka_ip: builtins.str
    kafka_port: builtins.int
    reload_enabled: builtins.bool
    @property
    def metricbeat_modules(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        kibana_ip: builtins.str = ...,
        kibana_port: builtins.int = ...,
        elastic_ip: builtins.str = ...,
        elastic_port: builtins.int = ...,
        num_elastic_shards: builtins.int = ...,
        kafka_ip: builtins.str = ...,
        kafka_port: builtins.int = ...,
        reload_enabled: builtins.bool = ...,
        metricbeat_modules: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["elastic_ip", b"elastic_ip", "elastic_port", b"elastic_port", "kafka_ip", b"kafka_ip", "kafka_port", b"kafka_port", "kibana_ip", b"kibana_ip", "kibana_port", b"kibana_port", "metricbeat_modules", b"metricbeat_modules", "num_elastic_shards", b"num_elastic_shards", "reload_enabled", b"reload_enabled"]) -> None: ...

global___ConfigMetricbeatMsg = ConfigMetricbeatMsg

@typing_extensions.final
class GetHostStatusMsg(google.protobuf.message.Message):
    """Message that the client sends to get the status of the HostMonitor"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___GetHostStatusMsg = GetHostStatusMsg

@typing_extensions.final
class HostStatusDTO(google.protobuf.message.Message):
    """Message that the server returns when asked about the status of running processes"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MONITOR_RUNNING_FIELD_NUMBER: builtins.int
    FILEBEAT_RUNNING_FIELD_NUMBER: builtins.int
    PACKETBEAT_RUNNING_FIELD_NUMBER: builtins.int
    METRICBEAT_RUNNING_FIELD_NUMBER: builtins.int
    HEARTBEAT_RUNNING_FIELD_NUMBER: builtins.int
    monitor_running: builtins.bool
    filebeat_running: builtins.bool
    packetbeat_running: builtins.bool
    metricbeat_running: builtins.bool
    heartbeat_running: builtins.bool
    def __init__(
        self,
        *,
        monitor_running: builtins.bool = ...,
        filebeat_running: builtins.bool = ...,
        packetbeat_running: builtins.bool = ...,
        metricbeat_running: builtins.bool = ...,
        heartbeat_running: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["filebeat_running", b"filebeat_running", "heartbeat_running", b"heartbeat_running", "metricbeat_running", b"metricbeat_running", "monitor_running", b"monitor_running", "packetbeat_running", b"packetbeat_running"]) -> None: ...

global___HostStatusDTO = HostStatusDTO

@typing_extensions.final
class GetHostMetricsMsg(google.protobuf.message.Message):
    """Message that the client sends to extract the Host metrics from given timestamps"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FAILED_AUTH_LAST_TS_FIELD_NUMBER: builtins.int
    LOGIN_LAST_TS_FIELD_NUMBER: builtins.int
    failed_auth_last_ts: builtins.float
    login_last_ts: builtins.float
    def __init__(
        self,
        *,
        failed_auth_last_ts: builtins.float = ...,
        login_last_ts: builtins.float = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["failed_auth_last_ts", b"failed_auth_last_ts", "login_last_ts", b"login_last_ts"]) -> None: ...

global___GetHostMetricsMsg = GetHostMetricsMsg

@typing_extensions.final
class HostMetricsDTO(google.protobuf.message.Message):
    """Message that the server returns when asked about the latest host metrics"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NUM_LOGGED_IN_USERS_FIELD_NUMBER: builtins.int
    NUM_FAILED_LOGIN_ATTEMPTS_FIELD_NUMBER: builtins.int
    NUM_OPEN_CONNECTIONS_FIELD_NUMBER: builtins.int
    NUM_LOGIN_EVENTS_FIELD_NUMBER: builtins.int
    NUM_PROCESSES_FIELD_NUMBER: builtins.int
    NUM_USERS_FIELD_NUMBER: builtins.int
    IP_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    num_logged_in_users: builtins.int
    num_failed_login_attempts: builtins.int
    num_open_connections: builtins.int
    num_login_events: builtins.int
    num_processes: builtins.int
    num_users: builtins.int
    ip: builtins.str
    timestamp: builtins.float
    def __init__(
        self,
        *,
        num_logged_in_users: builtins.int = ...,
        num_failed_login_attempts: builtins.int = ...,
        num_open_connections: builtins.int = ...,
        num_login_events: builtins.int = ...,
        num_processes: builtins.int = ...,
        num_users: builtins.int = ...,
        ip: builtins.str = ...,
        timestamp: builtins.float = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["ip", b"ip", "num_failed_login_attempts", b"num_failed_login_attempts", "num_logged_in_users", b"num_logged_in_users", "num_login_events", b"num_login_events", "num_open_connections", b"num_open_connections", "num_processes", b"num_processes", "num_users", b"num_users", "timestamp", b"timestamp"]) -> None: ...

global___HostMetricsDTO = HostMetricsDTO

@typing_extensions.final
class StopHeartbeatMsg(google.protobuf.message.Message):
    """Message that the client sends to stop heartbeat"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StopHeartbeatMsg = StopHeartbeatMsg

@typing_extensions.final
class StartHeartbeatMsg(google.protobuf.message.Message):
    """Message that the client sends to start heartbeat"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StartHeartbeatMsg = StartHeartbeatMsg

@typing_extensions.final
class ConfigHeartbeatMsg(google.protobuf.message.Message):
    """Message that the client sends to set the heartbeat configuration"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KIBANA_IP_FIELD_NUMBER: builtins.int
    KIBANA_PORT_FIELD_NUMBER: builtins.int
    ELASTIC_IP_FIELD_NUMBER: builtins.int
    ELASTIC_PORT_FIELD_NUMBER: builtins.int
    NUM_ELASTIC_SHARDS_FIELD_NUMBER: builtins.int
    HOSTS_TO_MONITOR_FIELD_NUMBER: builtins.int
    kibana_ip: builtins.str
    kibana_port: builtins.int
    elastic_ip: builtins.str
    elastic_port: builtins.int
    num_elastic_shards: builtins.int
    @property
    def hosts_to_monitor(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        kibana_ip: builtins.str = ...,
        kibana_port: builtins.int = ...,
        elastic_ip: builtins.str = ...,
        elastic_port: builtins.int = ...,
        num_elastic_shards: builtins.int = ...,
        hosts_to_monitor: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["elastic_ip", b"elastic_ip", "elastic_port", b"elastic_port", "hosts_to_monitor", b"hosts_to_monitor", "kibana_ip", b"kibana_ip", "kibana_port", b"kibana_port", "num_elastic_shards", b"num_elastic_shards"]) -> None: ...

global___ConfigHeartbeatMsg = ConfigHeartbeatMsg



import threading
import logging
import subprocess
import requests
import time
import json
import csle_collector.constants.constants as constants


class FailureDetector(threading.Thread):
    """
    Thread representing a client
    """

    def __init__(self, sleep_time: int, ip: str, ryu_web_port: int, ryu_port: int, controller: str, kafka_ip: str,
                 kafka_port: int, time_step_len: int) -> None:
        """
        Initializes the failure detector

        :param sleep_time: the period to check  for failures
        :param ip: the ip of the host
        :param ryu_web_port: the Ryu web port
        :param ryu_port: the Ryu port
        :param controller: the Ryu controller module
        :param kafka_ip: the Kafka IP
        :param kafka_port: the Kafka port
        :param time_step_len: the time-step length for Kafka logging
        """
        threading.Thread.__init__(self)
        self.sleep_time = sleep_time
        self.ip = ip
        self.ryu_web_port = ryu_web_port
        self.ryu_port = ryu_port
        self.controller = controller
        self.kafka_ip = kafka_ip
        self.kafka_port = kafka_port
        self.time_step_len = time_step_len
        self.done = False

    def run(self) -> None:
        """
        The failure detector loop

        :return: None
        """
        self.done = False
        while not self.done:
            status_url = f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}{self.ip}:{self.ryu_web_port}" \
                         f"{constants.RYU.STATUS_PRODUCER_HTTP_RESOURCE}"
            try:
                requests.get(status_url, timeout=constants.RYU.REQUEST_TIMEOUT_S)
            except Exception:
                logging.info("Restarting Ryu..")
                cmd = constants.RYU.STOP_RYU_CONTROLLER
                logging.info(f"Stopping ryu with command: {cmd}")
                result = subprocess.run(cmd.split(" "), capture_output=True, text=True)
                logging.info(f"Stdout: {result.stdout}, stderr: {result.stderr}")
                cmd = constants.RYU.STOP_RYU_CONTROLLER_MANAGER
                logging.info(f"Stopping ryu with command: {cmd}")
                result = subprocess.run(cmd.split(" "), capture_output=True, text=True)
                logging.info(f"Stdout: {result.stdout}, stderr: {result.stderr}")
                cmd = constants.RYU.START_RYU_CONTROLLER.format(self.ryu_port, self.ryu_web_port, self.controller)
                logging.info(f"Starting RYU controller with command: {cmd}")
                subprocess.Popen(cmd, shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                time.sleep(5)
                start_url = f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}{self.ip}:{self.ryu_web_port}" \
                            f"{constants.RYU.START_PRODUCER_HTTP_RESOURCE}"
                logging.info(f"Starting the RYU monitor by sending a PUT request to: {start_url}")
                requests.put(start_url, data=json.dumps({constants.KAFKA.BOOTSTRAP_SERVERS_PROPERTY: self.kafka_ip,
                                                         constants.RYU.TIME_STEP_LEN_SECONDS: self.time_step_len}),
                             timeout=constants.RYU.REQUEST_TIMEOUT_S)
                logging.info("RYU monitor started")
            time.sleep(self.sleep_time)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: elk_manager.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11\x65lk_manager.proto\"\x12\n\x10StartLogstashMsg\"\x10\n\x0eStartKibanaMsg\"\x11\n\x0fStartElasticMsg\"\x11\n\x0fStopLogstashMsg\"\x0f\n\rStopKibanaMsg\"\x10\n\x0eStopElasticMsg\"\x0c\n\nStopElkMsg\"\r\n\x0bStartElkMsg\"\x11\n\x0fGetElkStatusMsg\"P\n\x06\x45lkDTO\x12\x16\n\x0e\x65lasticRunning\x18\x01 \x01(\x08\x12\x15\n\rkibanaRunning\x18\x02 \x01(\x08\x12\x17\n\x0flogstashRunning\x18\x03 \x01(\x08\x32\x89\x03\n\nElkManager\x12+\n\x0cgetElkStatus\x12\x10.GetElkStatusMsg\x1a\x07.ElkDTO\"\x00\x12!\n\x07stopElk\x12\x0b.StopElkMsg\x1a\x07.ElkDTO\"\x00\x12#\n\x08startElk\x12\x0c.StartElkMsg\x1a\x07.ElkDTO\"\x00\x12)\n\x0bstopElastic\x12\x0f.StopElasticMsg\x1a\x07.ElkDTO\"\x00\x12+\n\x0cstartElastic\x12\x10.StartElasticMsg\x1a\x07.ElkDTO\"\x00\x12+\n\x0cstopLogstash\x12\x10.StopLogstashMsg\x1a\x07.ElkDTO\"\x00\x12-\n\rstartLogstash\x12\x11.StartLogstashMsg\x1a\x07.ElkDTO\"\x00\x12\'\n\nstopKibana\x12\x0e.StopKibanaMsg\x1a\x07.ElkDTO\"\x00\x12)\n\x0bstartKibana\x12\x0f.StartKibanaMsg\x1a\x07.ElkDTO\"\x00\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'elk_manager_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _STARTLOGSTASHMSG._serialized_start=21
  _STARTLOGSTASHMSG._serialized_end=39
  _STARTKIBANAMSG._serialized_start=41
  _STARTKIBANAMSG._serialized_end=57
  _STARTELASTICMSG._serialized_start=59
  _STARTELASTICMSG._serialized_end=76
  _STOPLOGSTASHMSG._serialized_start=78
  _STOPLOGSTASHMSG._serialized_end=95
  _STOPKIBANAMSG._serialized_start=97
  _STOPKIBANAMSG._serialized_end=112
  _STOPELASTICMSG._serialized_start=114
  _STOPELASTICMSG._serialized_end=130
  _STOPELKMSG._serialized_start=132
  _STOPELKMSG._serialized_end=144
  _STARTELKMSG._serialized_start=146
  _STARTELKMSG._serialized_end=159
  _GETELKSTATUSMSG._serialized_start=161
  _GETELKSTATUSMSG._serialized_end=178
  _ELKDTO._serialized_start=180
  _ELKDTO._serialized_end=260
  _ELKMANAGER._serialized_start=263
  _ELKMANAGER._serialized_end=656
# @@protoc_insertion_point(module_scope)



from typing import List, Dict, Any, Tuple
import datetime
import subprocess
from csle_collector.snort_ids_manager.dao.snort_ids_alert import SnortIdsAlert
from csle_collector.snort_ids_manager.dao.snort_ids_fast_log_alert import SnortIdsFastLogAlert
from csle_collector.snort_ids_manager.dao.snort_ids_alert_counters import SnortIdsAlertCounters
from csle_collector.snort_ids_manager.dao.snort_ids_rule_counters import SnortIdsRuleCounters
from csle_collector.snort_ids_manager.dao.snort_ids_ip_alert_counters import SnortIdsIPAlertCounters
import csle_collector.snort_ids_manager.snort_ids_manager_pb2
import csle_collector.constants.constants as constants


class SnortIdsManagerUtil:
    """
    Class with utility functions related to the Snort IDS Manager
    """

    @staticmethod
    def check_snort_ids_alerts() -> List[SnortIdsAlert]:
        """
        Reads alerts from the Snort IDS alerts log

        :return: a list of alerts
        """
        cmd = constants.SNORT_IDS_ROUTER.TAIL_ALERTS_COMMAND + " " + constants.SNORT_IDS_ROUTER.SNORT_ALERTS_FILE
        result = subprocess.run(cmd.split(" "), capture_output=True, text=True)
        alerts = []
        year = datetime.datetime.now().year
        for line in result.stdout.split("\n"):
            a_str = line.replace("\n", "")
            alerts.append(SnortIdsAlert.parse_from_str(a_str, year=year))
        return alerts

    @staticmethod
    def check_snort_ids_fast_log() -> List[SnortIdsFastLogAlert]:
        """
        Reads alerts from the Snort IDS fast-log

        :param env_config: the environment config
        :return: a list of alerts
        """
        cmd = constants.SNORT_IDS_ROUTER.TAIL_FAST_LOG_COMMAND + " " + constants.SNORT_IDS_ROUTER.SNORT_FAST_LOG_FILE
        result = subprocess.run(cmd.split(" "), capture_output=True, text=True)
        fast_logs = []
        year = datetime.datetime.now().year
        for line in result.stdout.split("\n"):
            if line is not None and line != "" and line != " ":
                a_str = line.replace("\n", "")
                fast_logs.append(SnortIdsAlert.fast_log_parse(a_str, year=year))
        return fast_logs

    @staticmethod
    def get_latest_snort_alert_ts() -> float:
        """
        Gets the latest timestamp in the snort alerts log

        :param env_config: the environment config
        :return: the latest timestamp
        """
        cmd = constants.SNORT_IDS_ROUTER.TAIL_ALERTS_LATEST_COMMAND + " " + constants.SNORT_IDS_ROUTER.SNORT_ALERTS_FILE
        result = subprocess.run(cmd.split(" "), capture_output=True, text=True)
        alerts = []
        year = datetime.datetime.now().year
        for line in result.stdout.split("\n"):
            if line != "" and line is not None and line != " ":
                a_str = line.replace("\n", "")
                alerts.append(SnortIdsAlert.parse_from_str(a_str, year=year))
        if len(alerts) == 0:
            # retry once
            result = subprocess.run(cmd.split(" "), capture_output=True, text=True)
            alerts = []
            for line in result.stdout.split("\n"):
                if line != "" and line is not None and line != " ":
                    a_str = line.replace("\n", "")
                    alerts.append(SnortIdsAlert.parse_from_str(a_str, year=year))
            if len(alerts) == 0:
                return datetime.datetime.now().timestamp()
            else:
                if alerts[0].timestamp is None:
                    raise ValueError("timestamp is None")
                return float(alerts[0].timestamp)
        else:
            if alerts[0].timestamp is None:
                raise ValueError("timestamp is None")
            return float(alerts[0].timestamp)

    @staticmethod
    def read_snort_ids_data(episode_last_alert_ts: float) -> \
            Tuple[SnortIdsAlertCounters, SnortIdsRuleCounters, List[SnortIdsIPAlertCounters]]:
        """
        Measures metrics from the Snort ids

        :param env_config: environment configuration
        :param episode_last_alert_ts: timestamp when the episode started
        :return: ids statistics
        """

        # Read Snort IDS data
        # alerts = IdsManagerUtil.check_ids_alerts()
        fast_logs = SnortIdsManagerUtil.check_snort_ids_fast_log()

        # Filter IDS data from beginning of episode
        # alerts = list(filter(lambda x: x.timestamp > episode_last_alert_ts, alerts))
        fast_logs = list(filter(lambda x: x.timestamp > episode_last_alert_ts, fast_logs))

        agg_counters = SnortIdsAlertCounters()
        agg_counters.count(fast_logs)
        rule_counters = SnortIdsRuleCounters()
        rule_counters.count(fast_logs)
        ip_counters = []
        ips = set()
        for fl in fast_logs:
            ips.add(fl.target_ip)
        for ip in ips:
            ip_counter = SnortIdsIPAlertCounters()
            ip_counter.alert_ip = ip
            ip_counter.count(fast_logs)
            ip_counters.append(ip_counter)
        return agg_counters, rule_counters, ip_counters

    @staticmethod
    def snort_ids_monitor_dto_to_dict(
            snort_ids_monitor_dto: csle_collector.snort_ids_manager.snort_ids_manager_pb2.SnortIdsMonitorDTO) \
            -> Dict[str, Any]:
        """
        Converts a SnortIDSMonitorDTO to a dict

        :param snort_ids_monitor_dto: the dto to convert
        :return: a dict representation of the DTO
        """
        d = {}
        d["monitor_running"] = snort_ids_monitor_dto.monitor_running
        d["snort_ids_running"] = snort_ids_monitor_dto.snort_ids_running
        return d

    @staticmethod
    def snort_ids_monitor_dto_from_dict(d: Dict[str, Any]) \
            -> csle_collector.snort_ids_manager.snort_ids_manager_pb2.SnortIdsMonitorDTO:
        """
        Converts a dict representation of a SnortIDSMonitorDTO to a DTO

        :param d: the dict to convert
        :return: the converted DTO
        """
        snort_ids_monitor_dto = csle_collector.snort_ids_manager.snort_ids_manager_pb2.SnortIdsMonitorDTO()
        snort_ids_monitor_dto.monitor_running = d["monitor_running"]
        snort_ids_monitor_dto.snort_ids_running = d["snort_ids_running"]
        return snort_ids_monitor_dto

    @staticmethod
    def snort_ids_log_dto_to_dict(
            snort_ids_log_dto: csle_collector.snort_ids_manager.snort_ids_manager_pb2.SnortIdsLogDTO) \
            -> Dict[str, Any]:
        """
        Converts a SnortIdsLogDTO to a dict

        :param snort_ids_log_dto: the DTO to convert
        :return: a dict representation of the DTO
        """
        d: Dict[str, Any] = {}
        d["timestamp"] = snort_ids_log_dto.timestamp
        d["ip"] = snort_ids_log_dto.ip
        d["attempted_admin_alerts"] = snort_ids_log_dto.attempted_admin_alerts
        d["attempted_user_alerts"] = snort_ids_log_dto.attempted_user_alerts
        d["inappropriate_content_alerts"] = snort_ids_log_dto.inappropriate_content_alerts
        d["policy_violation_alerts"] = snort_ids_log_dto.policy_violation_alerts
        d["shellcode_detect_alerts"] = snort_ids_log_dto.shellcode_detect_alerts
        d["successful_admin_alerts"] = snort_ids_log_dto.successful_admin_alerts
        d["successful_user_alerts"] = snort_ids_log_dto.successful_user_alerts
        d["trojan_activity_alerts"] = snort_ids_log_dto.trojan_activity_alerts
        d["unsuccessful_user_alerts"] = snort_ids_log_dto.unsuccessful_user_alerts
        d["web_application_attack_alerts"] = snort_ids_log_dto.web_application_attack_alerts
        d["attempted_dos_alerts"] = snort_ids_log_dto.attempted_dos_alerts
        d["attempted_recon_alerts"] = snort_ids_log_dto.attempted_recon_alerts
        d["bad_unknown_alerts"] = snort_ids_log_dto.bad_unknown_alerts
        d["default_login_attempt_alerts"] = snort_ids_log_dto.default_login_attempt_alerts
        d["denial_of_service_alerts"] = snort_ids_log_dto.denial_of_service_alerts
        d["misc_attack_alerts"] = snort_ids_log_dto.misc_attack_alerts
        d["non_standard_protocol_alerts"] = snort_ids_log_dto.non_standard_protocol_alerts
        d["rpc_portman_decode_alerts"] = snort_ids_log_dto.rpc_portman_decode_alerts
        d["successful_dos_alerts"] = snort_ids_log_dto.successful_dos_alerts
        d["successful_recon_largescale_alerts"] = snort_ids_log_dto.successful_recon_largescale_alerts
        d["successful_recon_limited_alerts"] = snort_ids_log_dto.successful_recon_limited_alerts
        d["suspicious_filename_detect_alerts"] = snort_ids_log_dto.suspicious_filename_detect_alerts
        d["suspicious_login_alerts"] = snort_ids_log_dto.suspicious_login_alerts
        d["system_call_detect_alerts"] = snort_ids_log_dto.system_call_detect_alerts
        d["unusual_client_port_connection_alerts"] = snort_ids_log_dto.unusual_client_port_connection_alerts
        d["web_application_activity_alerts"] = snort_ids_log_dto.web_application_activity_alerts
        d["icmp_event_alerts"] = snort_ids_log_dto.icmp_event_alerts
        d["misc_activity_alerts"] = snort_ids_log_dto.misc_activity_alerts
        d["network_scan_alerts"] = snort_ids_log_dto.network_scan_alerts
        d["not_suspicious_alerts"] = snort_ids_log_dto.not_suspicious_alerts
        d["protocol_command_decode_alerts"] = snort_ids_log_dto.protocol_command_decode_alerts
        d["unknown_alerts"] = snort_ids_log_dto.unknown_alerts
        d["tcp_connection_alerts"] = snort_ids_log_dto.tcp_connection_alerts
        d["priority_1_alerts"] = snort_ids_log_dto.priority_1_alerts
        d["priority_2_alerts"] = snort_ids_log_dto.priority_2_alerts
        d["priority_3_alerts"] = snort_ids_log_dto.priority_3_alerts
        d["priority_4_alerts"] = snort_ids_log_dto.priority_4_alerts
        d["total_alerts"] = snort_ids_log_dto.total_alerts
        d["warning_alerts"] = snort_ids_log_dto.warning_alerts
        d["severe_alerts"] = snort_ids_log_dto.severe_alerts
        d["alerts_weighted_by_priority"] = snort_ids_log_dto.alerts_weighted_by_priority
        return d

    @staticmethod
    def snort_ids_log_dto_from_dict(d: Dict[str, Any]) \
            -> csle_collector.snort_ids_manager.snort_ids_manager_pb2.SnortIdsLogDTO:
        """
        Converts a dict representation of a SnortIdsLogDTO to a DTO

        :param d: the dict to convert
        :return: the converted DTO
        """
        snort_ids_log_dto = csle_collector.snort_ids_manager.snort_ids_manager_pb2.SnortIdsLogDTO()
        snort_ids_log_dto.timestamp = d["timestamp"]
        snort_ids_log_dto.ip = d["ip"]
        snort_ids_log_dto.attempted_admin_alerts = d["attempted_admin_alerts"]
        snort_ids_log_dto.attempted_user_alerts = d["attempted_user_alerts"]
        snort_ids_log_dto.inappropriate_content_alerts = d["inappropriate_content_alerts"]
        snort_ids_log_dto.policy_violation_alerts = d["policy_violation_alerts"]
        snort_ids_log_dto.shellcode_detect_alerts = d["shellcode_detect_alerts"]
        snort_ids_log_dto.successful_admin_alerts = d["successful_admin_alerts"]
        snort_ids_log_dto.successful_user_alerts = d["successful_user_alerts"]
        snort_ids_log_dto.trojan_activity_alerts = d["trojan_activity_alerts"]
        snort_ids_log_dto.unsuccessful_user_alerts = d["unsuccessful_user_alerts"]
        snort_ids_log_dto.web_application_attack_alerts = d["web_application_attack_alerts"]
        snort_ids_log_dto.attempted_dos_alerts = d["attempted_dos_alerts"]
        snort_ids_log_dto.attempted_recon_alerts = d["attempted_recon_alerts"]
        snort_ids_log_dto.bad_unknown_alerts = d["bad_unknown_alerts"]
        snort_ids_log_dto.default_login_attempt_alerts = d["default_login_attempt_alerts"]
        snort_ids_log_dto.denial_of_service_alerts = d["denial_of_service_alerts"]
        snort_ids_log_dto.misc_attack_alerts = d["misc_attack_alerts"]
        snort_ids_log_dto.non_standard_protocol_alerts = d["non_standard_protocol_alerts"]
        snort_ids_log_dto.rpc_portman_decode_alerts = d["rpc_portman_decode_alerts"]
        snort_ids_log_dto.successful_dos_alerts = d["successful_dos_alerts"]
        snort_ids_log_dto.successful_recon_largescale_alerts = d["successful_recon_largescale_alerts"]
        snort_ids_log_dto.successful_recon_limited_alerts = d["successful_recon_limited_alerts"]
        snort_ids_log_dto.suspicious_filename_detect_alerts = d["suspicious_filename_detect_alerts"]
        snort_ids_log_dto.suspicious_login_alerts = d["suspicious_login_alerts"]
        snort_ids_log_dto.system_call_detect_alerts = d["system_call_detect_alerts"]
        snort_ids_log_dto.unusual_client_port_connection_alerts = d["unusual_client_port_connection_alerts"]
        snort_ids_log_dto.web_application_activity_alerts = d["web_application_activity_alerts"]
        snort_ids_log_dto.icmp_event_alerts = d["icmp_event_alerts"]
        snort_ids_log_dto.misc_activity_alerts = d["misc_activity_alerts"]
        snort_ids_log_dto.network_scan_alerts = d["network_scan_alerts"]
        snort_ids_log_dto.not_suspicious_alerts = d["not_suspicious_alerts"]
        snort_ids_log_dto.protocol_command_decode_alerts = d["protocol_command_decode_alerts"]
        snort_ids_log_dto.unknown_alerts = d["unknown_alerts"]
        snort_ids_log_dto.tcp_connection_alerts = d["tcp_connection_alerts"]
        snort_ids_log_dto.priority_1_alerts = d["priority_1_alerts"]
        snort_ids_log_dto.priority_2_alerts = d["priority_2_alerts"]
        snort_ids_log_dto.priority_3_alerts = d["priority_3_alerts"]
        snort_ids_log_dto.priority_4_alerts = d["priority_4_alerts"]
        snort_ids_log_dto.total_alerts = d["total_alerts"]
        snort_ids_log_dto.warning_alerts = d["warning_alerts"]
        snort_ids_log_dto.severe_alerts = d["severe_alerts"]
        snort_ids_log_dto.alerts_weighted_by_priority = d["alerts_weighted_by_priority"]
        return snort_ids_log_dto

    @staticmethod
    def snort_ids_log_dto_empty() \
            -> csle_collector.snort_ids_manager.snort_ids_manager_pb2.SnortIdsLogDTO:
        """
        :return: an empty SnortIdsLogDTO
        """
        snort_ids_log_dto = csle_collector.snort_ids_manager.snort_ids_manager_pb2.SnortIdsLogDTO()
        snort_ids_log_dto.timestamp = 0.0
        snort_ids_log_dto.ip = ""
        snort_ids_log_dto.attempted_admin_alerts = 0
        snort_ids_log_dto.attempted_user_alerts = 0
        snort_ids_log_dto.inappropriate_content_alerts = 0
        snort_ids_log_dto.policy_violation_alerts = 0
        snort_ids_log_dto.shellcode_detect_alerts = 0
        snort_ids_log_dto.successful_admin_alerts = 0
        snort_ids_log_dto.successful_user_alerts = 0
        snort_ids_log_dto.trojan_activity_alerts = 0
        snort_ids_log_dto.unsuccessful_user_alerts = 0
        snort_ids_log_dto.web_application_attack_alerts = 0
        snort_ids_log_dto.attempted_dos_alerts = 0
        snort_ids_log_dto.attempted_recon_alerts = 0
        snort_ids_log_dto.bad_unknown_alerts = 0
        snort_ids_log_dto.default_login_attempt_alerts = 0
        snort_ids_log_dto.denial_of_service_alerts = 0
        snort_ids_log_dto.misc_attack_alerts = 0
        snort_ids_log_dto.non_standard_protocol_alerts = 0
        snort_ids_log_dto.rpc_portman_decode_alerts = 0
        snort_ids_log_dto.successful_dos_alerts = 0
        snort_ids_log_dto.successful_recon_largescale_alerts = 0
        snort_ids_log_dto.successful_recon_limited_alerts = 0
        snort_ids_log_dto.suspicious_filename_detect_alerts = 0
        snort_ids_log_dto.suspicious_login_alerts = 0
        snort_ids_log_dto.system_call_detect_alerts = 0
        snort_ids_log_dto.unusual_client_port_connection_alerts = 0
        snort_ids_log_dto.web_application_activity_alerts = 0
        snort_ids_log_dto.icmp_event_alerts = 0
        snort_ids_log_dto.misc_activity_alerts = 0
        snort_ids_log_dto.network_scan_alerts = 0
        snort_ids_log_dto.not_suspicious_alerts = 0
        snort_ids_log_dto.protocol_command_decode_alerts = 0
        snort_ids_log_dto.unknown_alerts = 0
        snort_ids_log_dto.tcp_connection_alerts = 0
        snort_ids_log_dto.priority_1_alerts = 0
        snort_ids_log_dto.priority_2_alerts = 0
        snort_ids_log_dto.priority_3_alerts = 0
        snort_ids_log_dto.priority_4_alerts = 0
        snort_ids_log_dto.total_alerts = 0
        snort_ids_log_dto.warning_alerts = 0
        snort_ids_log_dto.severe_alerts = 0
        snort_ids_log_dto.alerts_weighted_by_priority = 0
        return snort_ids_log_dto

    @staticmethod
    def snort_ids_monitor_dto_empty() -> csle_collector.snort_ids_manager.snort_ids_manager_pb2.SnortIdsMonitorDTO:
        """
        :return: An empty SnortIdsMonitorDTO
        """
        snort_ids_monitor_dto = csle_collector.snort_ids_manager.snort_ids_manager_pb2.SnortIdsMonitorDTO()
        snort_ids_monitor_dto.monitor_running = False
        snort_ids_monitor_dto.snort_ids_running = False
        return snort_ids_monitor_dto



import time
import logging
import threading
from confluent_kafka import Producer
from csle_collector.host_manager.host_manager_util import HostManagerUtil
import csle_collector.constants.constants as constants


class HostMonitorThread(threading.Thread):
    """
    Thread that collects the Host statistics and pushes it to Kafka periodically
    """

    def __init__(self, kafka_ip: str, kafka_port: int, ip: str, hostname: str, time_step_len_seconds: int) -> None:
        """
        Initializes the thread

        :param kafka_ip: IP of the Kafka server to push to
        :param kafka_port: port of the Kafka server to push to
        :param ip: ip of the server we are pushing from
        :param hostname: hostname of the server we are pushing from
        :param time_step_len_seconds: the length of a timestep
        """
        threading.Thread.__init__(self)
        self.kafka_ip = kafka_ip
        self.kafka_port = kafka_port
        self.ip = ip
        self.hostname = hostname
        self.latest_ts = time.time()
        self.failed_auth_last_ts = HostManagerUtil.read_latest_ts_auth()
        self.login_last_ts = HostManagerUtil.read_latest_ts_login()
        self.time_step_len_seconds = time_step_len_seconds
        self.conf = {
            constants.KAFKA.BOOTSTRAP_SERVERS_PROPERTY: f"{self.kafka_ip}:{self.kafka_port}",
            constants.KAFKA.CLIENT_ID_PROPERTY: self.hostname}
        self.producer = Producer(**self.conf)
        self.running = True
        logging.info("HostMonitor thread started successfully")

    def run(self) -> None:
        """
        Main loop of the thread. Parses log files and metrics on the host and pushes it to Kafka periodically

        :return: None
        """
        logging.info("HostMonitor [Running]")
        while self.running:
            time.sleep(self.time_step_len_seconds)
            try:
                host_metrics = HostManagerUtil.read_host_metrics(failed_auth_last_ts=self.failed_auth_last_ts,
                                                                 login_last_ts=self.login_last_ts)
                record = host_metrics.to_kafka_record(ip=self.ip)
                self.producer.produce(constants.KAFKA_CONFIG.HOST_METRICS_TOPIC_NAME, record)
                self.producer.poll(0)
                self.failed_auth_last_ts = HostManagerUtil.read_latest_ts_auth()
                self.login_last_ts = HostManagerUtil.read_latest_ts_login()
            except Exception as e:
                logging.info(f"[Monitor thread], "
                             f"There was an exception reading host metrics and producing to kafka: "
                             f"{str(e)}, {repr(e)}")



from typing import List, Dict, Any, Tuple, Union
import csle_collector.client_manager.client_manager_pb2
from csle_base.json_serializable import JSONSerializable
from csle_base.grpc_serializable import GRPCSerializable


class WorkflowService(JSONSerializable, GRPCSerializable):
    """
    A service of the network.
    The service might be distributed across several network nodes.
    The service is defined by the series of commands that a client executes to make use of the service.
    """

    def __init__(self, ips_and_commands: List[Tuple[str, List[str]]], id: int) -> None:
        """
        Initializes the object

        :param id: the id of the service
        :param ips_and_commands: the list of commands
        """
        self.ips_and_commands = ips_and_commands
        self.id = id

    @staticmethod
    def from_dict(d: Dict[str, Any]) -> "WorkflowService":
        """
        Converts a dict representation to an instance

        :param d: the dict to convert
        :return: the created instance
        """
        obj = WorkflowService(ips_and_commands=list(map(lambda x: (x[0], x[1]),
                                                        d["ips_and_commands"])), id=d["id"])
        return obj

    def to_dict(self) -> Dict[str, Any]:
        """
        Converts the object to a dict representation
        
        :return: a dict representation of the object
        """
        d: Dict[str, Union[int, List[Tuple[str, List[str]]]]] = {}
        d["ips_and_commands"] = self.ips_and_commands
        d["id"] = self.id
        return d

    @staticmethod
    def from_json_file(json_file_path: str) -> "WorkflowService":
        """
        Reads a json file and converts it to a DTO

        :param json_file_path: the json file path
        :return: the converted DTO
        """
        import io
        import json
        with io.open(json_file_path, 'r') as f:
            json_str = f.read()
        return WorkflowService.from_dict(json.loads(json_str))

    def copy(self) -> "WorkflowService":
        """
        :return: a copy of the DTO
        """
        return WorkflowService.from_dict(self.to_dict())

    @staticmethod
    def replace_first_octet_of_ip(ip: str, ip_first_octet: int) -> str:
        """
        Utility function for changing the first octet in an IP address

        :param ip: the IP to modify
        :param ip_first_octet: the first octet to insert
        :return: the new IP
        """
        index_of_first_octet_end = ip.find(".")
        return str(ip_first_octet) + ip[index_of_first_octet_end:]

    def create_execution_config(self, ip_first_octet: int) -> "WorkflowService":
        """
        Creates a new config for an execution

        :param ip_first_octet: the first octet of the IP of the new execution
        :return: the new config
        """
        config = self.copy()
        for i in range(len(config.ips_and_commands)):
            config.ips_and_commands[i] = \
                (WorkflowService.replace_first_octet_of_ip(ip=config.ips_and_commands[i][0],
                                                           ip_first_octet=ip_first_octet),
                 config.ips_and_commands[i][1])
        return config

    def to_grpc_object(self) -> csle_collector.client_manager.client_manager_pb2.WorkflowServiceDTO:
        """
        :return: a GRPC serializable version of the object
        """
        ips = []
        commands = []
        for i in range(len(self.ips_and_commands)):
            ips.append(self.ips_and_commands[i][0])
            commands.append(csle_collector.client_manager.client_manager_pb2.NodeCommandsDTO(
                commands=self.ips_and_commands[i][1]))
        return csle_collector.client_manager.client_manager_pb2.WorkflowServiceDTO(
            id=self.id, ips=ips, commands=commands)

    @staticmethod
    def from_grpc_object(obj: csle_collector.client_manager.client_manager_pb2.WorkflowServiceDTO) \
            -> "WorkflowService":
        """
        Instantiates the object from a GRPC DTO

        :param obj: the object to instantiate from
        :return: the instantiated object
        """
        ips_and_commands = []
        ips = list(obj.ips)
        commands = list(obj.commands)
        for i in range(len(list(ips))):
            ips_and_commands.append((ips[i], list(commands[i].commands)))
        return WorkflowService(id=obj.id, ips_and_commands=ips_and_commands)

    def get_commands(self) -> List[str]:
        """
        :return: the list of commands for the service
        """
        commands = []
        for i in range(len(self.ips_and_commands)):
            for j in range(len(self.ips_and_commands[i][1])):
                commands.append(self.ips_and_commands[i][1][j].format(self.ips_and_commands[i][0]))
        return commands

    def __str__(self) -> str:
        """
        :return: a string representation of the object
        """
        return f"Workflow service, id: {self.id}, ips and commands: {self.ips_and_commands}"



from typing import Dict, Any, Tuple, List, Union
import time
from csle_base.json_serializable import JSONSerializable
import csle_collector.host_manager.host_manager_pb2


class HostMetrics(JSONSerializable):
    """
    DTO class containing host metrics
    """

    def __init__(self, num_logged_in_users: int = 0, num_failed_login_attempts: int = 0,
                 num_open_connections: int = 0,
                 num_login_events: int = 0, num_processes: int = 0, num_users: int = 0,
                 ip: Union[None, str] = None, ts: Union[float, None] = None) -> None:
        """
        Initializes the DTO

        :param num_logged_in_users: the number of logged in users
        :param num_failed_login_attempts: the number of failed login attempts
        :param num_open_connections: the number of open connections
        :param num_login_events: the number of login events
        :param num_processes: the number of processes
        :param num_users: the number of users
        """
        self.num_logged_in_users = num_logged_in_users
        self.num_failed_login_attempts = num_failed_login_attempts
        self.num_open_connections = num_open_connections
        self.num_login_events = num_login_events
        self.num_processes = num_processes
        self.num_users = num_users
        self.ts = ts
        self.ip = ip

    def to_dto(self, ip: str) -> csle_collector.host_manager.host_manager_pb2.HostMetricsDTO:
        """
        Converts the object into a gRPC DTO for serialization

        :param ip: the ip to add to the DTO in addition to the metrics
        :return: a csle_collector.host_manager.host_manager_pb2.HostMetricsDTO
        """
        ts = time.time()
        return csle_collector.host_manager.host_manager_pb2.HostMetricsDTO(
            num_logged_in_users=self.num_logged_in_users, num_failed_login_attempts=self.num_failed_login_attempts,
            num_open_connections=self.num_open_connections, num_login_events=self.num_login_events,
            num_processes=self.num_processes, num_users=self.num_users, ip=ip, timestamp=ts)

    def to_kafka_record(self, ip: str) -> str:
        """
        Converts the DTO into a Kafka record string

        :param ip: the IP to add to the record in addition to the metrics
        :return: a comma separated string representing the kafka record
        """
        ts = time.time()
        record_str = f"{ts},{ip},{self.num_logged_in_users},{self.num_failed_login_attempts}," \
                     f"{self.num_open_connections},{self.num_login_events},{self.num_processes},{self.num_users}"
        return record_str

    @staticmethod
    def from_kafka_record(record: str) -> "HostMetrics":
        """
        Converts the Kafka record string to a DTO

        :param record: the kafka record
        :return: the created DTO
        """
        parts = record.split(",")
        obj = HostMetrics(
            ip=parts[1], ts=float(parts[0]), num_logged_in_users=int(parts[2]), num_failed_login_attempts=int(parts[3]),
            num_open_connections=int(parts[4]), num_login_events=int(parts[5]), num_processes=int(parts[6]),
            num_users=int(parts[7]))
        return obj

    def update_with_kafka_record(self, record: str, ip: str) -> None:
        """
        Updates the DTO based on a kafka record

        :param record: the kafka record
        :param ip: the host ip
        :return: None
        """
        parts = record.split(",")
        if parts[1] == ip:
            self.ip = parts[1]
            self.ts = float(parts[0])
            self.num_logged_in_users = int(parts[2])
            self.num_failed_login_attempts = int(parts[3])
            self.num_open_connections = int(parts[4])
            self.num_login_events = int(parts[5])
            self.num_processes = int(parts[6])
            self.num_users = int(parts[7])

    def __str__(self) -> str:
        """
        :return: a string representation of the object
        """
        return f"num_logged_in_users:{self.num_logged_in_users}, " \
               f"num_failed_login_attempts: {self.num_failed_login_attempts}, " \
               f"num_open_connections:{self.num_open_connections}, " \
               f"num_login_events:{self.num_login_events}, num_processes: {self.num_processes}," \
               f"num_users: {self.num_users}"

    @staticmethod
    def from_dict(d: Dict[str, Any]) -> "HostMetrics":
        """
        Converts a dict representation to an instance

        :param d: the dict to convert
        :return: the created instance
        """
        obj = HostMetrics(
            num_logged_in_users=d["num_logged_in_users"],
            num_failed_login_attempts=d["num_failed_login_attempts"],
            num_open_connections=d["num_open_connections"],
            num_login_events=d["num_login_events"],
            num_processes=d["num_processes"],
            num_users=d["num_users"],
            ip=d["ip"],
            ts=d["ts"],
        )
        return obj

    def to_dict(self) -> Dict[str, Any]:
        """
        :return: a dict representation of the instance
        """
        d: Dict[str, Any] = {}
        d["num_logged_in_users"] = self.num_logged_in_users
        d["num_failed_login_attempts"] = self.num_failed_login_attempts
        d["num_open_connections"] = self.num_open_connections
        d["num_login_events"] = self.num_login_events
        d["num_processes"] = self.num_processes
        d["num_users"] = self.num_users
        d["ts"] = self.ts
        d["ip"] = self.ip
        return d

    def copy(self) -> "HostMetrics":
        """
        :return: a copy of the object
        """
        c = HostMetrics(
            num_logged_in_users=self.num_logged_in_users, num_failed_login_attempts=self.num_failed_login_attempts,
            num_open_connections=self.num_open_connections, num_login_events=self.num_login_events,
            num_processes=self.num_processes, num_users=self.num_users, ip=self.ip, ts=self.ts
        )
        return c

    def get_deltas(self, stats_prime: "HostMetrics") -> Tuple[List[int], List[str]]:
        """
        Get the deltas between two stats objects

        :param stats_prime: the stats object to compare with
        :param max_counter: the maximum counter_value
        :return: the deltas and the labels
        """
        deltas = [
            int(stats_prime.num_logged_in_users - self.num_logged_in_users),
            int(stats_prime.num_failed_login_attempts - self.num_failed_login_attempts),
            int(stats_prime.num_open_connections - self.num_open_connections),
            int(stats_prime.num_login_events - self.num_login_events),
            int(stats_prime.num_processes - self.num_processes),
            int(stats_prime.num_users - self.num_users)
        ]
        labels = ["num_logged_in_users", "num_failed_login_attempts", "num_open_connections",
                  "num_login_events", "num_processes", "num_users"]
        return deltas, labels

    def get_values(self) -> Tuple[List[int], List[str]]:
        """
        Get the current values

        :return: the values and the labels
        """
        deltas = [
            int(self.num_logged_in_users),
            int(self.num_failed_login_attempts),
            int(self.num_open_connections),
            int(self.num_login_events),
            int(self.num_processes),
            int(self.num_users)
        ]
        labels = ["num_logged_in_users", "num_failed_login_attempts", "num_open_connections",
                  "num_login_events", "num_processes", "num_users"]
        return deltas, labels

    def num_attributes(self) -> int:
        """
        :return: The number of attributes of the DTO
        """
        return 8

    @staticmethod
    def schema() -> "HostMetrics":
        """
        :return: get the schema of the DTO
        """
        return HostMetrics()

    @staticmethod
    def from_json_file(json_file_path: str) -> "HostMetrics":
        """
        Reads a json file and converts it to a DTO

        :param json_file_path: the json file path
        :return: the converted DTO
        """
        import io
        import json
        with io.open(json_file_path, 'r') as f:
            json_str = f.read()
        return HostMetrics.from_dict(json.loads(json_str))



from typing import Dict, Any
import re
import datetime
from csle_base.json_serializable import JSONSerializable
from csle_collector.snort_ids_manager.dao.snort_ids_fast_log_alert import SnortIdsFastLogAlert
import csle_collector.constants.constants as constants


class SnortIdsAlert(JSONSerializable):
    """
    Object representing an IDS Alert
    """

    def __init__(self):
        """
        Initializes the IDS Alert Fields
        """
        self.timestamp = None
        self.sig_generator = None
        self.sig_id = None
        self.sig_rev = None
        self.msg = None
        self.proto = None
        self.src_ip = None
        self.src_port = None
        self.dst_ip = None
        self.dst_port = None
        self.eth_src = None
        self.eth_dst = None
        self.eth_len = None
        self.tcp_flags = None
        self.tcp_seq = None
        self.tcp_ack = None
        self.tcp_len = None
        self.tcp_window = None
        self.ttl = None
        self.tos = None
        self.id = None
        self.dgm_len = None
        self.ip_len = None
        self.icmp_type = None
        self.icmp_code = None
        self.icmp_id = None
        self.icmp_seq = None
        self.priority = None

    @staticmethod
    def parse_from_str(csv_str_record: str, year: int) -> "SnortIdsAlert":
        """
        Parses the IDS alert from a string

        :param csv_str_record: the string to parse
        :param year: the year of the entry
        :return: the parsed IDS Alert
        """
        a_fields = csv_str_record.split(",")
        alert_dao = SnortIdsAlert()
        if len(a_fields) > 1:
            alert_dao.timestamp = a_fields[0]
            if alert_dao.timestamp is not None and alert_dao.timestamp != "" and alert_dao.timestamp != "0":
                alert_dao.timestamp = str(year) + " " + alert_dao.timestamp
                try:
                    alert_dao.timestamp = datetime.datetime.strptime(alert_dao.timestamp.strip(),
                                                                     '%Y %m/%d-%H:%M:%S.%f').timestamp()
                except Exception:
                    alert_dao.timestamp = datetime.datetime.strptime("2010 04/20-08:46:14.094913",
                                                                     '%Y %m/%d-%H:%M:%S.%f').timestamp()
            else:
                alert_dao.timestamp = datetime.datetime.strptime("2010 04/20-08:46:14.094913",
                                                                 '%Y %m/%d-%H:%M:%S.%f').timestamp()
        else:
            alert_dao.timestamp = datetime.datetime.strptime("2010 04/20-08:46:14.094913",
                                                             '%Y %m/%d-%H:%M:%S.%f').timestamp()
        if len(a_fields) > 1:
            alert_dao.sig_generator = a_fields[1]
        if len(a_fields) > 2:
            alert_dao.sig_id = a_fields[2]
        if len(a_fields) > 3:
            alert_dao.sig_rev = a_fields[3]
        if len(a_fields) > 4:
            alert_dao.msg = a_fields[4]
        if len(a_fields) > 5:
            alert_dao.proto = a_fields[5]
        if len(a_fields) > 6:
            alert_dao.src_ip = a_fields[6]
        if len(a_fields) > 7:
            alert_dao.src_port = a_fields[7]
        if len(a_fields) > 8:
            alert_dao.dst_ip = a_fields[8]
        if len(a_fields) > 9:
            alert_dao.dst_port = a_fields[9]
        if len(a_fields) > 10:
            alert_dao.eth_src = a_fields[10]
        if len(a_fields) > 11:
            alert_dao.eth_dst = a_fields[11]
        if len(a_fields) > 12:
            alert_dao.eth_len = a_fields[12]
        if len(a_fields) > 13:
            alert_dao.tcp_flags = a_fields[13]
        if len(a_fields) > 14:
            alert_dao.tcp_seq = a_fields[14]
        if len(a_fields) > 15:
            alert_dao.tcp_ack = a_fields[15]
        if len(a_fields) > 16:
            alert_dao.tcp_len = a_fields[16]
        if len(a_fields) > 17:
            alert_dao.tcp_window = a_fields[17]
        if len(a_fields) > 18:
            alert_dao.ttl = a_fields[18]
        if len(a_fields) > 19:
            alert_dao.tos = a_fields[19]
        if len(a_fields) > 20:
            alert_dao.id = a_fields[20]
        if len(a_fields) > 21:
            alert_dao.dgm_len = a_fields[21]
        if len(a_fields) > 22:
            alert_dao.ip_len = a_fields[22]
        if len(a_fields) > 23:
            alert_dao.icmp_type = a_fields[23]
        if len(a_fields) > 24:
            alert_dao.icmp_code = a_fields[24]
        if len(a_fields) > 25:
            alert_dao.icmp_id = a_fields[25]
        if len(a_fields) > 26:
            alert_dao.icmp_seq = a_fields[26]
        alert_dao.priority = 1
        return alert_dao

    def set_priority(self, priority: int) -> None:
        """
        Sets the priority of the alert DTO

        :param priority: the priority to set
        :return: None
        """
        self.priority = priority

    @staticmethod
    def fast_log_parse(fast_log_str: str, year: int) -> SnortIdsFastLogAlert:
        """
        Parses the IDS Alert from a given string from the fast-log of Snort

        :param fast_log_str: the fast log string to parse
        :param year: the year
        :return: the priority, the class, and the time-stamp
        """
        priorities = re.findall(constants.SNORT_IDS_ROUTER.PRIORITY_REGEX, fast_log_str)
        if len(priorities) > 0:
            temp = priorities[0].replace("Priority: ", "")
            priority = int(temp)
        else:
            priority = 1
        alert_classes = re.findall(constants.SNORT_IDS_ROUTER.CLASSIFICATION_REGEX, fast_log_str)
        alert_class = "unknown"
        alert_class_id = 1
        if len(alert_classes) > 0:
            alert_class = alert_classes[0]
        if alert_class in constants.SNORT_IDS_ROUTER.SNORT_ALERT_IDS_ID:
            alert_class_id = constants.SNORT_IDS_ROUTER.SNORT_ALERT_IDS_ID[alert_class]

        ts_str = fast_log_str.split(" ")[0]
        if ts_str is not None and ts_str != "":
            ts_str = ts_str.strip()
            if ts_str != "":
                ts_str = str(year) + " " + ts_str
                try:
                    ts = datetime.datetime.strptime(ts_str.strip(), '%Y %m/%d-%H:%M:%S.%f').timestamp()
                except Exception:
                    ts = datetime.datetime.strptime("2010 04/20-08:46:14.094913", '%Y %m/%d-%H:%M:%S.%f').timestamp()
            else:
                ts = datetime.datetime.strptime("2010 04/20-08:46:14.094913", '%Y %m/%d-%H:%M:%S.%f').timestamp()

        source_ip = ""
        target_ip = ""
        ips_match = re.findall(constants.SNORT_IDS_ROUTER.IPS_REGEX, fast_log_str)
        if len(ips_match) > 0:
            ips = ips_match[0].replace(" ", "").split("->")
            source_ip = ips[0]
            target_ip = ips[1]

        rule_id = ""
        rule_match = re.findall(constants.SNORT_IDS_ROUTER.RULE_ID_REGEX, fast_log_str)
        if len(rule_match) > 0:
            rule_id = rule_match[0].replace("[", "").replace(":", "-")

        fast_log_alert = SnortIdsFastLogAlert(timestamp=ts, priority=priority, class_id=alert_class_id,
                                              source_ip=source_ip, target_ip=target_ip, rule_id=rule_id)
        return fast_log_alert

    def to_dict(self) -> Dict[str, Any]:
        """
        Converts the object to a dict representation

        :return: a dict representation of the object
        """
        d: Dict[str, Any] = {}
        d["timestamp"] = self.timestamp
        d["sig_generator"] = self.sig_generator
        d["sig_id"] = self.sig_id
        d["sig_rev"] = self.sig_rev
        d["msg"] = self.msg
        d["proto"] = self.proto
        d["src_ip"] = self.src_ip
        d["src_port"] = self.src_port
        d["dst_ip"] = self.dst_ip
        d["dst_port"] = self.dst_port
        d["eth_src"] = self.eth_src
        d["eth_dst"] = self.eth_dst
        d["eth_len"] = self.eth_len
        d["tcp_flags"] = self.tcp_flags
        d["tcp_seq"] = self.tcp_seq
        d["tcp_ack"] = self.tcp_ack
        d["tcp_len"] = self.tcp_len
        d["tcp_window"] = self.tcp_window
        d["ttl"] = self.ttl
        d["tos"] = self.tos
        d["id"] = self.id
        d["dgm_len"] = self.dgm_len
        d["ip_len"] = self.ip_len
        d["icmp_type"] = self.icmp_type
        d["icmp_code"] = self.icmp_code
        d["icmp_id"] = self.icmp_id
        d["icmp_seq"] = self.icmp_seq
        d["priority"] = self.priority
        return d

    @staticmethod
    def from_dict(parsed_stats_dict: Dict[str, Any]) -> "SnortIdsAlert":
        """
        Parses a SnortIdsAlert object from a dict

        :param parsed_stats_dict: the dict to parse
        :return: the parsed SnortIdsAlert object
        """
        snort_ids_alert = SnortIdsAlert()
        snort_ids_alert.timestamp = parsed_stats_dict["timestamp"]
        snort_ids_alert.sig_generator = parsed_stats_dict["sig_generator"]
        snort_ids_alert.sig_id = parsed_stats_dict["sig_id"]
        snort_ids_alert.sig_rev = parsed_stats_dict["sig_rev"]
        snort_ids_alert.msg = parsed_stats_dict["msg"]
        snort_ids_alert.proto = parsed_stats_dict["proto"]
        snort_ids_alert.src_ip = parsed_stats_dict["src_ip"]
        snort_ids_alert.src_port = parsed_stats_dict["src_port"]
        snort_ids_alert.dst_ip = parsed_stats_dict["dst_ip"]
        snort_ids_alert.dst_port = parsed_stats_dict["dst_port"]
        snort_ids_alert.eth_src = parsed_stats_dict["eth_src"]
        snort_ids_alert.eth_dst = parsed_stats_dict["eth_dst"]
        snort_ids_alert.eth_len = parsed_stats_dict["eth_len"]
        snort_ids_alert.tcp_flags = parsed_stats_dict["tcp_flags"]
        snort_ids_alert.tcp_seq = parsed_stats_dict["tcp_seq"]
        snort_ids_alert.tcp_ack = parsed_stats_dict["tcp_ack"]
        snort_ids_alert.tcp_len = parsed_stats_dict["tcp_len"]
        snort_ids_alert.tcp_window = parsed_stats_dict["tcp_window"]
        snort_ids_alert.ttl = parsed_stats_dict["ttl"]
        snort_ids_alert.tos = parsed_stats_dict["tos"]
        snort_ids_alert.id = parsed_stats_dict["id"]
        snort_ids_alert.dgm_len = parsed_stats_dict["dgm_len"]
        snort_ids_alert.ip_len = parsed_stats_dict["ip_len"]
        snort_ids_alert.icmp_type = parsed_stats_dict["icmp_type"]
        snort_ids_alert.icmp_code = parsed_stats_dict["icmp_code"]
        snort_ids_alert.icmp_id = parsed_stats_dict["icmp_id"]
        snort_ids_alert.icmp_seq = parsed_stats_dict["icmp_seq"]
        snort_ids_alert.priority = parsed_stats_dict["priority"]
        return snort_ids_alert

    @staticmethod
    def from_json_file(json_file_path: str) -> "SnortIdsAlert":
        """
        Reads a json file and converts it to a DTO

        :param json_file_path: the json file path
        :return: the converted DTO
        """
        import io
        import json
        with io.open(json_file_path, 'r') as f:
            json_str = f.read()
        return SnortIdsAlert.from_dict(json.loads(json_str))



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import csle_collector.traffic_manager.traffic_manager_pb2 as traffic__manager__pb2


class TrafficManagerStub(object):
    """Interface exported by the server
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.getTrafficStatus = channel.unary_unary(
                '/TrafficManager/getTrafficStatus',
                request_serializer=traffic__manager__pb2.GetTrafficStatusMsg.SerializeToString,
                response_deserializer=traffic__manager__pb2.TrafficDTO.FromString,
                )
        self.stopTraffic = channel.unary_unary(
                '/TrafficManager/stopTraffic',
                request_serializer=traffic__manager__pb2.StopTrafficMsg.SerializeToString,
                response_deserializer=traffic__manager__pb2.TrafficDTO.FromString,
                )
        self.startTraffic = channel.unary_unary(
                '/TrafficManager/startTraffic',
                request_serializer=traffic__manager__pb2.StartTrafficMsg.SerializeToString,
                response_deserializer=traffic__manager__pb2.TrafficDTO.FromString,
                )


class TrafficManagerServicer(object):
    """Interface exported by the server
    """

    def getTrafficStatus(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def stopTraffic(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def startTraffic(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_TrafficManagerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'getTrafficStatus': grpc.unary_unary_rpc_method_handler(
                    servicer.getTrafficStatus,
                    request_deserializer=traffic__manager__pb2.GetTrafficStatusMsg.FromString,
                    response_serializer=traffic__manager__pb2.TrafficDTO.SerializeToString,
            ),
            'stopTraffic': grpc.unary_unary_rpc_method_handler(
                    servicer.stopTraffic,
                    request_deserializer=traffic__manager__pb2.StopTrafficMsg.FromString,
                    response_serializer=traffic__manager__pb2.TrafficDTO.SerializeToString,
            ),
            'startTraffic': grpc.unary_unary_rpc_method_handler(
                    servicer.startTraffic,
                    request_deserializer=traffic__manager__pb2.StartTrafficMsg.FromString,
                    response_serializer=traffic__manager__pb2.TrafficDTO.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'TrafficManager', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class TrafficManager(object):
    """Interface exported by the server
    """

    @staticmethod
    def getTrafficStatus(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/TrafficManager/getTrafficStatus',
            traffic__manager__pb2.GetTrafficStatusMsg.SerializeToString,
            traffic__manager__pb2.TrafficDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def stopTraffic(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/TrafficManager/stopTraffic',
            traffic__manager__pb2.StopTrafficMsg.SerializeToString,
            traffic__manager__pb2.TrafficDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def startTraffic(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/TrafficManager/startTraffic',
            traffic__manager__pb2.StartTrafficMsg.SerializeToString,
            traffic__manager__pb2.TrafficDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



from typing import List, Union
import logging
import socket
import netifaces
import time
import grpc
import os
from concurrent import futures
import subprocess
import csle_collector.host_manager.host_manager_pb2_grpc
import csle_collector.host_manager.host_manager_pb2
from csle_collector.host_manager.host_manager_util import HostManagerUtil
from csle_collector.host_manager.threads.host_monitor_thread import HostMonitorThread
import csle_collector.constants.constants as constants


class HostManagerServicer(csle_collector.host_manager.host_manager_pb2_grpc.HostManagerServicer):
    """
    gRPC server for collecting Host statistics.
    """

    def __init__(self) -> None:
        """
        Initializes the server
        """
        logging.basicConfig(filename=f"{constants.LOG_FILES.HOST_MANAGER_LOG_DIR}"
                                     f"{constants.LOG_FILES.HOST_MANAGER_LOG_FILE}", level=logging.INFO)
        self.hostname = socket.gethostname()
        try:
            self.ip = netifaces.ifaddresses(constants.INTERFACES.ETH0)[netifaces.AF_INET][0][constants.INTERFACES.ADDR]
        except Exception:
            self.ip = socket.gethostbyname(self.hostname)
        self.conf = {constants.KAFKA.BOOTSTRAP_SERVERS_PROPERTY: f"{self.ip}:{constants.KAFKA.PORT}",
                     constants.KAFKA.CLIENT_ID_PROPERTY: self.hostname}
        self.host_monitor_thread: Union[None, HostMonitorThread] = None
        logging.info(f"Starting the HostManager hostname: {self.hostname} ip: {self.ip}")

    def getHostMetrics(self, request: csle_collector.host_manager.host_manager_pb2.GetHostMetricsMsg,
                       context: grpc.ServicerContext) -> csle_collector.host_manager.host_manager_pb2.HostMetricsDTO:
        """
        Gets the host metrics from given timestamps

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with IDS statistics
        """
        host_metrics = HostManagerUtil.read_host_metrics(failed_auth_last_ts=request.failed_auth_last_ts,
                                                         login_last_ts=request.login_last_ts)
        host_metrics_dto = host_metrics.to_dto(ip=self.ip)
        return host_metrics_dto

    def startHostMonitor(self, request: csle_collector.host_manager.host_manager_pb2.StartHostMonitorMsg,
                         context: grpc.ServicerContext) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
        """
        Starts the Host monitor thread

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with the status of the Host monitor thread
        """
        logging.info(f"Starting the HostMonitor thread, timestep length: {request.time_step_len_seconds}, "
                     f"kafka ip: {request.kafka_ip}, "
                     f"kafka port: {request.kafka_port}")
        if self.host_monitor_thread is not None:
            self.host_monitor_thread.running = False
        self.host_monitor_thread = HostMonitorThread(kafka_ip=request.kafka_ip, kafka_port=request.kafka_port,
                                                     ip=self.ip, hostname=self.hostname,
                                                     time_step_len_seconds=request.time_step_len_seconds)
        self.host_monitor_thread.start()
        logging.info("Started the HostMonitor thread")
        filebeat_status = HostManagerServicer._get_filebeat_status()
        packetbeat_status = HostManagerServicer._get_packetbeat_status()
        metricbeat_status = HostManagerServicer._get_metricbeat_status()
        heartbeat_status = HostManagerServicer._get_heartbeat_status()
        return csle_collector.host_manager.host_manager_pb2.HostStatusDTO(monitor_running=True,
                                                                          filebeat_running=filebeat_status,
                                                                          packetbeat_running=packetbeat_status,
                                                                          metricbeat_running=metricbeat_status,
                                                                          heartbeat_running=heartbeat_status)

    def stopHostMonitor(self, request: csle_collector.host_manager.host_manager_pb2.StopHostMonitorMsg,
                        context: grpc.ServicerContext) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
        """
        Stops the Host monitor thread if it is running

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with the status of the Host monitor thread
        """
        logging.info("Stopping the host monitor")
        if self.host_monitor_thread is not None:
            self.host_monitor_thread.running = False
        logging.info("Host monitor stopped")
        filebeat_running = HostManagerServicer._get_filebeat_status()
        packetbeat_status = HostManagerServicer._get_packetbeat_status()
        metricbeat_status = HostManagerServicer._get_metricbeat_status()
        heartbeat_status = HostManagerServicer._get_heartbeat_status()
        return csle_collector.host_manager.host_manager_pb2.HostStatusDTO(monitor_running=False,
                                                                          filebeat_running=filebeat_running,
                                                                          packetbeat_running=packetbeat_status,
                                                                          metricbeat_running=metricbeat_status,
                                                                          heartbeat_running=heartbeat_status)

    def startFilebeat(self, request: csle_collector.host_manager.host_manager_pb2.StartFilebeatMsg,
                      context: grpc.ServicerContext) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
        """
        Starts filebeat

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with the status of the Host
        """
        logging.info("Starting filebeat")
        HostManagerServicer._start_filebeat()
        logging.info("Started filebeat")
        monitor_running = self._is_monitor_running()
        packetbeat_status = HostManagerServicer._get_packetbeat_status()
        metricbeat_status = HostManagerServicer._get_metricbeat_status()
        heartbeat_status = HostManagerServicer._get_heartbeat_status()
        return csle_collector.host_manager.host_manager_pb2.HostStatusDTO(monitor_running=monitor_running,
                                                                          filebeat_running=True,
                                                                          packetbeat_running=packetbeat_status,
                                                                          metricbeat_running=metricbeat_status,
                                                                          heartbeat_running=heartbeat_status)

    def stopFilebeat(self, request: csle_collector.host_manager.host_manager_pb2.StopFilebeatMsg,
                     context: grpc.ServicerContext) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
        """
        Stops filebeat

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with the status of the Host
        """
        logging.info("Stopping filebeat")
        HostManagerServicer._stop_filebeat()
        logging.info("Filebeat stopped")
        monitor_running = self._is_monitor_running()
        packetbeat_status = HostManagerServicer._get_packetbeat_status()
        metricbeat_status = HostManagerServicer._get_metricbeat_status()
        heartbeat_status = HostManagerServicer._get_heartbeat_status()
        return csle_collector.host_manager.host_manager_pb2.HostStatusDTO(monitor_running=monitor_running,
                                                                          filebeat_running=False,
                                                                          packetbeat_running=packetbeat_status,
                                                                          metricbeat_running=metricbeat_status,
                                                                          heartbeat_running=heartbeat_status)

    def configFilebeat(self, request: csle_collector.host_manager.host_manager_pb2.ConfigFilebeatMsg,
                       context: grpc.ServicerContext) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
        """
        Updates the configuration of filebeat

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with the status of the Host
        """
        logging.info(f"Updating the filebeat configuration, "
                     f"log_files_paths: {request.log_files_paths}, "
                     f"kibana_ip : {request.kibana_ip}, kibana_port: {request.kibana_port}, "
                     f"elastic_ip: {request.elastic_ip}, elastic_port: {request.elastic_port}, "
                     f"num_elastic_shards: {request.num_elastic_shards}, reload_enabled: {request.reload_enabled}, "
                     f"kafka: {request.kafka}, kafka_ip: {request.kafka_ip}, kafka_port: {request.kafka_port}, "
                     f"kafka_topics: {request.kafka_topics}, filebeat_modules: {request.filebeat_modules}")
        HostManagerServicer._set_filebeat_config(
            log_files_paths=list(request.log_files_paths), kibana_ip=request.kibana_ip, kibana_port=request.kibana_port,
            elastic_ip=request.elastic_ip, elastic_port=request.elastic_port,
            num_elastic_shards=request.num_elastic_shards, reload_enabled=request.reload_enabled,
            kafka=request.kafka, kafka_ip=request.kafka_ip, kafka_port=request.kafka_port,
            kafka_topics=list(request.kafka_topics), filebeat_modules=list(request.filebeat_modules))
        logging.info("Filebeat configuration updated")
        monitor_running = self._is_monitor_running()
        filebeat_running = HostManagerServicer._get_filebeat_status()
        packetbeat_status = HostManagerServicer._get_packetbeat_status()
        metricbeat_status = HostManagerServicer._get_metricbeat_status()
        heartbeat_status = HostManagerServicer._get_heartbeat_status()
        return csle_collector.host_manager.host_manager_pb2.HostStatusDTO(monitor_running=monitor_running,
                                                                          filebeat_running=filebeat_running,
                                                                          packetbeat_running=packetbeat_status,
                                                                          metricbeat_running=metricbeat_status,
                                                                          heartbeat_running=heartbeat_status)

    def startMetricbeat(self, request: csle_collector.host_manager.host_manager_pb2.StartMetricbeatMsg,
                        context: grpc.ServicerContext) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
        """
        Starts metricbeat

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with the status of the Host
        """
        logging.info("Starting metricbeat")
        HostManagerServicer._start_metricbeat()
        logging.info("Started metricbeat")
        monitor_running = self._is_monitor_running()
        packetbeat_status = HostManagerServicer._get_packetbeat_status()
        filebeat_status = HostManagerServicer._get_filebeat_status()
        heartbeat_status = HostManagerServicer._get_heartbeat_status()
        return csle_collector.host_manager.host_manager_pb2.HostStatusDTO(monitor_running=monitor_running,
                                                                          filebeat_running=filebeat_status,
                                                                          packetbeat_running=packetbeat_status,
                                                                          metricbeat_running=True,
                                                                          heartbeat_running=heartbeat_status)

    def stopMetricbeat(self, request: csle_collector.host_manager.host_manager_pb2.StopMetricbeatMsg,
                       context: grpc.ServicerContext) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
        """
        Stops metricbeat

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with the status of the Host
        """
        logging.info("Stopping metricbeat")
        HostManagerServicer._stop_metricbeat()
        logging.info("Metricbeat stopped")
        monitor_running = self._is_monitor_running()
        packetbeat_status = HostManagerServicer._get_packetbeat_status()
        filebeat_status = HostManagerServicer._get_filebeat_status()
        heartbeat_status = HostManagerServicer._get_heartbeat_status()
        return csle_collector.host_manager.host_manager_pb2.HostStatusDTO(monitor_running=monitor_running,
                                                                          filebeat_running=filebeat_status,
                                                                          packetbeat_running=packetbeat_status,
                                                                          metricbeat_running=False,
                                                                          heartbeat_running=heartbeat_status)

    def configMetricbeat(self, request: csle_collector.host_manager.host_manager_pb2.ConfigMetricbeatMsg,
                         context: grpc.ServicerContext) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
        """
        Updates the configuration of metricbeat

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with the status of the Host
        """
        logging.info(f"Updating the metricbeat configuration, "
                     f"kibana_ip : {request.kibana_ip}, kibana_port: {request.kibana_port}, "
                     f"elastic_ip: {request.elastic_ip}, elastic_port: {request.elastic_port}, "
                     f"num_elastic_shards: {request.num_elastic_shards}, reload_enabled: {request.reload_enabled}, "
                     f"kafka_ip: {request.kafka_ip}, kafka_port: {request.kafka_port}, "
                     f"metricbeat_modules: {request.metricbeat_modules}")
        HostManagerServicer._set_metricbeat_config(
            kibana_ip=request.kibana_ip, kibana_port=request.kibana_port,
            elastic_ip=request.elastic_ip, elastic_port=request.elastic_port,
            num_elastic_shards=request.num_elastic_shards, reload_enabled=request.reload_enabled,
            kafka_ip=request.kafka_ip, kafka_port=request.kafka_port,
            metricbeat_modules=list(request.metricbeat_modules))
        logging.info("Metricbeat configuration updated")
        monitor_running = self._is_monitor_running()
        filebeat_running = HostManagerServicer._get_filebeat_status()
        packetbeat_status = HostManagerServicer._get_packetbeat_status()
        metricbeat_status = HostManagerServicer._get_metricbeat_status()
        heartbeat_status = HostManagerServicer._get_heartbeat_status()
        return csle_collector.host_manager.host_manager_pb2.HostStatusDTO(monitor_running=monitor_running,
                                                                          filebeat_running=filebeat_running,
                                                                          packetbeat_running=packetbeat_status,
                                                                          metricbeat_running=metricbeat_status,
                                                                          heartbeat_running=heartbeat_status)

    def startPacketbeat(self, request: csle_collector.host_manager.host_manager_pb2.StartPacketbeatMsg,
                        context: grpc.ServicerContext) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
        """
        Starts packetbeat

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with the status of the Host
        """
        logging.info("Starting packetbeat")
        HostManagerServicer._start_packetbeat()
        logging.info("Started packetbeat")
        monitor_running = self._is_monitor_running()
        filebeat_status = HostManagerServicer._get_filebeat_status()
        metricbeat_status = HostManagerServicer._get_metricbeat_status()
        heartbeat_status = HostManagerServicer._get_heartbeat_status()
        return csle_collector.host_manager.host_manager_pb2.HostStatusDTO(monitor_running=monitor_running,
                                                                          filebeat_running=filebeat_status,
                                                                          packetbeat_running=True,
                                                                          metricbeat_running=metricbeat_status,
                                                                          heartbeat_running=heartbeat_status)

    def stopPacketbeat(self, request: csle_collector.host_manager.host_manager_pb2.StopPacketbeatMsg,
                       context: grpc.ServicerContext) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
        """
        Stops packetbeat

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with the status of the Host
        """
        logging.info("Stopping packetbeat")
        HostManagerServicer._stop_packetbeat()
        logging.info("Packetbeat stopped")
        monitor_running = self._is_monitor_running()
        filebeat_status = HostManagerServicer._get_filebeat_status()
        metricbeat_status = HostManagerServicer._get_metricbeat_status()
        heartbeat_status = HostManagerServicer._get_heartbeat_status()
        return csle_collector.host_manager.host_manager_pb2.HostStatusDTO(monitor_running=monitor_running,
                                                                          filebeat_running=filebeat_status,
                                                                          packetbeat_running=False,
                                                                          metricbeat_running=metricbeat_status,
                                                                          heartbeat_running=heartbeat_status)

    def configPacketbeat(self, request: csle_collector.host_manager.host_manager_pb2.ConfigPacketbeatMsg,
                         context: grpc.ServicerContext) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
        """
        Updates the configuration of packetbeat

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with the status of the Host
        """
        logging.info(f"Updating the packetbeat configuration,"
                     f"kibana_ip : {request.kibana_ip}, kibana_port: {request.kibana_port}, "
                     f"elastic_ip: {request.elastic_ip}, elastic_port: {request.elastic_port}, "
                     f"num_elastic_shards: {request.num_elastic_shards}")
        HostManagerServicer._set_packetbeat_config(
            kibana_ip=request.kibana_ip, kibana_port=request.kibana_port,
            elastic_ip=request.elastic_ip, elastic_port=request.elastic_port,
            num_elastic_shards=request.num_elastic_shards)
        logging.info("Packetbeat configuration updated")
        monitor_running = self._is_monitor_running()
        filebeat_running = HostManagerServicer._get_filebeat_status()
        packetbeat_status = HostManagerServicer._get_packetbeat_status()
        metricbeat_status = HostManagerServicer._get_metricbeat_status()
        heartbeat_status = HostManagerServicer._get_heartbeat_status()
        return csle_collector.host_manager.host_manager_pb2.HostStatusDTO(monitor_running=monitor_running,
                                                                          filebeat_running=filebeat_running,
                                                                          packetbeat_running=packetbeat_status,
                                                                          metricbeat_running=metricbeat_status,
                                                                          heartbeat_running=heartbeat_status)

    def getHostStatus(self, request: csle_collector.host_manager.host_manager_pb2.GetHostStatusMsg,
                      context: grpc.ServicerContext) \
            -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
        """
        Gets the status of the host

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with the status of the host
        """
        monitor_running = self._is_monitor_running()
        filebeat_running = HostManagerServicer._get_filebeat_status()
        packetbeat_status = HostManagerServicer._get_packetbeat_status()
        metricbeat_status = HostManagerServicer._get_metricbeat_status()
        heartbeat_status = HostManagerServicer._get_heartbeat_status()
        return csle_collector.host_manager.host_manager_pb2.HostStatusDTO(monitor_running=monitor_running,
                                                                          filebeat_running=filebeat_running,
                                                                          packetbeat_running=packetbeat_status,
                                                                          metricbeat_running=metricbeat_status,
                                                                          heartbeat_running=heartbeat_status)

    def startHeartbeat(self, request: csle_collector.host_manager.host_manager_pb2.StartHeartbeatMsg,
                       context: grpc.ServicerContext) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
        """
        Starts heartbeat

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with the status of the Host
        """
        logging.info("Starting heartbeat")
        HostManagerServicer._start_heartbeat()
        logging.info("Started heartbeat")
        monitor_running = self._is_monitor_running()
        packetbeat_status = HostManagerServicer._get_packetbeat_status()
        filebeat_status = HostManagerServicer._get_filebeat_status()
        metricbeat_status = HostManagerServicer._get_metricbeat_status()
        return csle_collector.host_manager.host_manager_pb2.HostStatusDTO(monitor_running=monitor_running,
                                                                          filebeat_running=filebeat_status,
                                                                          packetbeat_running=packetbeat_status,
                                                                          metricbeat_running=metricbeat_status,
                                                                          heartbeat_running=True)

    def stopHeartbeat(self, request: csle_collector.host_manager.host_manager_pb2.StopHeartbeatMsg,
                      context: grpc.ServicerContext) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
        """
        Stops heartbeat

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with the status of the Host
        """
        logging.info("Stopping heartbeat")
        HostManagerServicer._stop_heartbeat()
        logging.info("Heartbeat stopped")
        monitor_running = self._is_monitor_running()
        packetbeat_status = HostManagerServicer._get_packetbeat_status()
        filebeat_status = HostManagerServicer._get_filebeat_status()
        metricbeat_status = HostManagerServicer._get_metricbeat_status()
        return csle_collector.host_manager.host_manager_pb2.HostStatusDTO(monitor_running=monitor_running,
                                                                          filebeat_running=filebeat_status,
                                                                          packetbeat_running=packetbeat_status,
                                                                          metricbeat_running=metricbeat_status,
                                                                          heartbeat_running=False)

    def configHeartbeat(self, request: csle_collector.host_manager.host_manager_pb2.ConfigHeartbeatMsg,
                        context: grpc.ServicerContext) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
        """
        Updates the configuration of heartbeat

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with the status of the Host
        """
        logging.info(f"Updating the heartbeat configuration, "
                     f"kibana_ip : {request.kibana_ip}, kibana_port: {request.kibana_port}, "
                     f"elastic_ip: {request.elastic_ip}, elastic_port: {request.elastic_port}, "
                     f"num_elastic_shards: {request.num_elastic_shards}, hosts_to_monitor: {request.hosts_to_monitor}")
        HostManagerServicer._set_heartbeat_config(
            kibana_ip=request.kibana_ip, kibana_port=request.kibana_port,
            elastic_ip=request.elastic_ip, elastic_port=request.elastic_port,
            num_elastic_shards=request.num_elastic_shards, hosts_to_monitor=list(request.hosts_to_monitor))
        logging.info("Heartbeat configuration updated")
        monitor_running = self._is_monitor_running()
        filebeat_running = HostManagerServicer._get_filebeat_status()
        packetbeat_status = HostManagerServicer._get_packetbeat_status()
        metricbeat_status = HostManagerServicer._get_metricbeat_status()
        heartbeat_status = HostManagerServicer._get_heartbeat_status()
        return csle_collector.host_manager.host_manager_pb2.HostStatusDTO(monitor_running=monitor_running,
                                                                          filebeat_running=filebeat_running,
                                                                          packetbeat_running=packetbeat_status,
                                                                          metricbeat_running=metricbeat_status,
                                                                          heartbeat_running=heartbeat_status)

    def startSpark(self, request: csle_collector.host_manager.host_manager_pb2.StartSparkMsg,
                   context: grpc.ServicerContext) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
        """
        Starts Spark

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with the status of the Host
        """
        logging.info("Starting spark")
        HostManagerServicer._start_spark()
        logging.info("Started spark")
        monitor_running = self._is_monitor_running()
        packetbeat_status = HostManagerServicer._get_packetbeat_status()
        filebeat_status = HostManagerServicer._get_filebeat_status()
        metricbeat_status = HostManagerServicer._get_metricbeat_status()
        heartbeat_status = HostManagerServicer._get_heartbeat_status()
        return csle_collector.host_manager.host_manager_pb2.HostStatusDTO(monitor_running=monitor_running,
                                                                          filebeat_running=filebeat_status,
                                                                          packetbeat_running=packetbeat_status,
                                                                          metricbeat_running=metricbeat_status,
                                                                          heartbeat_running=heartbeat_status)

    def stopSpark(self, request: csle_collector.host_manager.host_manager_pb2.StopSparkMsg,
                  context: grpc.ServicerContext) -> csle_collector.host_manager.host_manager_pb2.HostStatusDTO:
        """
        Stops Spark

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with the status of the Host
        """
        logging.info("Stopping spark")
        HostManagerServicer._stop_spark()
        logging.info("Stopped spark")
        monitor_running = self._is_monitor_running()
        packetbeat_status = HostManagerServicer._get_packetbeat_status()
        filebeat_status = HostManagerServicer._get_filebeat_status()
        metricbeat_status = HostManagerServicer._get_metricbeat_status()
        heartbeat_status = HostManagerServicer._get_heartbeat_status()
        return csle_collector.host_manager.host_manager_pb2.HostStatusDTO(monitor_running=monitor_running,
                                                                          filebeat_running=filebeat_status,
                                                                          packetbeat_running=packetbeat_status,
                                                                          metricbeat_running=metricbeat_status,
                                                                          heartbeat_running=heartbeat_status)

    def _is_monitor_running(self) -> bool:
        """
        Utility method to check if the monitor is running

        :return: True if running else false
        """
        if self.host_monitor_thread is not None:
            return self.host_monitor_thread.running
        return False

    @staticmethod
    def _get_filebeat_status() -> bool:
        """
        Utility method to get the status of filebeat

        :return: status of filebeat
        """
        logging.info(f"Getting filebeat status with command: {constants.FILEBEAT.FILEBEAT_STATUS}")
        output = subprocess.run(constants.FILEBEAT.FILEBEAT_STATUS.split(" "), capture_output=True, text=True)
        filebeat_running = not ("not" in output.stdout)
        logging.info(f"Got filebeat status, output:{output.stdout}, err output: {output.stderr} ")
        return filebeat_running

    @staticmethod
    def _start_filebeat() -> None:
        """
        Utility method to start filebeat

        :return: None
        """
        logging.info(f"Starting filebeat with command: {constants.FILEBEAT.FILEBEAT_START}")
        output = subprocess.run(constants.FILEBEAT.FILEBEAT_START.split(" "), capture_output=True, text=True)
        logging.info(f"Started filebeat, stdout:{output.stdout}, stderr: {output.stderr}")

    @staticmethod
    def _stop_filebeat() -> None:
        """
        Utility method to stop filebeat

        :return: None
        """
        logging.info(f"Stopping filebeat with command: {constants.FILEBEAT.FILEBEAT_STOP}")
        output = subprocess.run(constants.FILEBEAT.FILEBEAT_STOP.split(" "), capture_output=True, text=True)
        logging.info(f"Stopped filebeat, output:{output.stdout}, err output: {output.stderr} ")

    @staticmethod
    def _set_filebeat_config(log_files_paths: List[str], kibana_ip: str, kibana_port: int, elastic_ip: str,
                             elastic_port: int, num_elastic_shards: int, kafka_topics: List[str], kafka_ip: str,
                             filebeat_modules: List[str],
                             kafka_port: int, reload_enabled: bool = False, kafka: bool = False) -> None:
        """
        Updates the filebeat configuration file

        :param log_files_paths: the list of log files that filebeat should monitor
        :param kibana_ip: the IP of Kibana where the data should be visualized
        :param kibana_port: the port of Kibana where the data should be visualized
        :param elastic_ip: the IP of elastic where the data should be shipped
        :param elastic_port: the port of elastic where the data should be shipped
        :param num_elastic_shards: the number of elastic shards
        :param reload_enabled: whether automatic reload of modules should be enabled
        :param kafka: whether kafka should be added as input
        :param kafka_topics: list of kafka topics to ingest
        :param kafka_port: the kafka server port
        :param kafka_ip: the kafka server ip
        :param filebeat_modules: list of filebeat modules to enable
        :return: None
        """
        filebeat_config = HostManagerUtil.filebeat_config(log_files_paths=log_files_paths, kibana_ip=kibana_ip,
                                                          kibana_port=kibana_port, elastic_ip=elastic_ip,
                                                          elastic_port=elastic_port,
                                                          num_elastic_shards=num_elastic_shards,
                                                          reload_enabled=reload_enabled, kafka=kafka,
                                                          kafka_port=kafka_port, kafka_ip=kafka_ip,
                                                          kafka_topics=kafka_topics)
        for module in filebeat_modules:
            if module == constants.FILEBEAT.SYSTEM_MODULE:
                HostManagerServicer.set_filebeat_system_module_config()
            elif module == constants.FILEBEAT.SNORT_MODULE:
                HostManagerServicer.set_filebeat_snort_module_config()
            elif module == constants.FILEBEAT.KAFKA_MODULE:
                HostManagerServicer.set_filebeat_kafka_module_config()
            elif module == constants.FILEBEAT.KIBANA_MODULE:
                HostManagerServicer.set_filebeat_kibana_module_config()
            elif module == constants.FILEBEAT.ELASTICSEARCH_MODULE:
                HostManagerServicer.set_filebeat_elasticsearch_module_config()
            elif module == constants.FILEBEAT.LOGSTASH_MODULE:
                HostManagerServicer.set_filebeat_logstash_module_config()
            else:
                logging.warning(f"Filebeat module: {module} not recognized")

        logging.info(f"Updating filebeat config: \n{filebeat_config}")
        HostManagerUtil.write_yaml_config(config=filebeat_config, path=constants.FILEBEAT.CONFIG_FILE)
        logging.info(f"Running filebeat setup command: {constants.FILEBEAT.SETUP_CMD}")
        output = subprocess.run(constants.FILEBEAT.SETUP_CMD.split(" "), capture_output=True, text=True)
        logging.info(f"Stdout of the setup command: {output.stdout}, stderr of the setup command: {output.stderr}")

    @staticmethod
    def _get_metricbeat_status() -> bool:
        """
        Utility method to get the status of metricbeat

        :return: status of metricbeat
        """
        logging.info(f"Getting metricbeat status with command: {constants.METRICBEAT.METRICBEAT_STATUS}")
        output = subprocess.run(constants.METRICBEAT.METRICBEAT_STATUS.split(" "), capture_output=True, text=True)
        metricbeat_running = not ("not" in output.stdout)
        logging.info(f"Got metricbeat status, output:{output.stdout}, err output: {output.stderr} ")
        return metricbeat_running

    @staticmethod
    def _start_metricbeat() -> None:
        """
        Utility method to start metricbeat

        :return: None
        """
        logging.info(f"Starting metricbeat with command: {constants.METRICBEAT.METRICBEAT_START}")
        output = subprocess.run(constants.METRICBEAT.METRICBEAT_START.split(" "), capture_output=True, text=True)
        logging.info(f"Started metricbeat, stdout:{output.stdout}, stderr: {output.stderr}")

    @staticmethod
    def _stop_metricbeat() -> None:
        """
        Utility method to stop metricbeat

        :return: None
        """
        logging.info(f"Stopping metricbeat with command: {constants.METRICBEAT.METRICBEAT_STOP}")
        output = subprocess.run(constants.METRICBEAT.METRICBEAT_STOP.split(" "), capture_output=True, text=True)
        logging.info(f"Stopped metricbeat, output:{output.stdout}, err output: {output.stderr} ")

    @staticmethod
    def _set_metricbeat_config(
            kibana_ip: str, kibana_port: int, elastic_ip: str, elastic_port: int, num_elastic_shards: int,
            metricbeat_modules: List[str], kafka_ip: str, kafka_port: int, reload_enabled: bool = False) -> None:
        """
        Updates the metricbeat configuration file

        :param kibana_ip: the IP of Kibana where the data should be visualized
        :param kibana_port: the port of Kibana where the data should be visualized
        :param elastic_ip: the IP of elastic where the data should be shipped
        :param elastic_port: the port of elastic where the data should be shipped
        :param num_elastic_shards: the number of elastic shards
        :param reload_enabled: whether automatic reload of modules should be enabled
        :param metricbeat_modules: list of metricbeat modules to enable
        :param kafka_ip: the ip of the kafka server
        :param kafka_port: the port of the kafka server
        :return: None
        """
        metricbeat_config = HostManagerUtil.metricbeat_config(
            kibana_ip=kibana_ip, kibana_port=kibana_port, elastic_ip=elastic_ip, elastic_port=elastic_port,
            num_elastic_shards=num_elastic_shards, reload_enabled=reload_enabled)
        for module in metricbeat_modules:
            if module == constants.METRICBEAT.SYSTEM_MODULE:
                HostManagerServicer.set_metricbeat_system_module_config()
            elif module == constants.METRICBEAT.LINUX_MODULE:
                HostManagerServicer.set_metricbeat_linux_module_config()
            elif module == constants.METRICBEAT.KAFKA_MODULE:
                HostManagerServicer.set_metricbeat_kafka_module_config(kafka_ip=kafka_ip, kafka_port=kafka_port)
            elif module == constants.METRICBEAT.KIBANA_MODULE:
                HostManagerServicer.set_metricbeat_kibana_module_config(kibana_ip=kibana_ip, kibana_port=kibana_port)
            elif module == constants.METRICBEAT.ELASTICSEARCH_MODULE:
                HostManagerServicer.set_metricbeat_elasticsearch_module_config(elastic_ip=elastic_ip,
                                                                               elastic_port=elastic_port)
            elif module == constants.METRICBEAT.LOGSTASH_MODULE:
                HostManagerServicer.set_metricbeat_logstash_module_config(logstash_ip=elastic_ip,
                                                                          logstash_port=elastic_port)
            else:
                logging.warning(f"Metricbeat module: {module} not recognized")

        logging.info(f"Updating metricbeat config: \n{metricbeat_config}")
        HostManagerUtil.write_yaml_config(config=metricbeat_config, path=constants.METRICBEAT.CONFIG_FILE)
        logging.info(f"Running metricbeat setup command: {constants.METRICBEAT.SETUP_CMD}")
        output = subprocess.run(constants.METRICBEAT.SETUP_CMD.split(" "), capture_output=True, text=True)
        logging.info(f"Stdout of the setup command: {output.stdout}, stderr of the setup command: {output.stderr}")

    @staticmethod
    def set_filebeat_snort_module_config() -> None:
        """
        Updates the filebeat snort module configuration

        :return: None
        """
        logging.info(f"Enabling module with command: "
                     f"{constants.FILEBEAT.ENABLE_MODULE_CMD.format(constants.FILEBEAT.SNORT_MODULE)}")
        output = subprocess.run(constants.FILEBEAT.ENABLE_MODULE_CMD.format(constants.FILEBEAT.SNORT_MODULE).split(" "),
                                capture_output=True, text=True)
        logging.info(f"Module enabled, output: {output.stdout}, err output: {output.stderr}")
        snort_module_config = HostManagerUtil.filebeat_snort_module_config()
        logging.info(f"Updating filebeat snort module config: \n{snort_module_config}")
        HostManagerUtil.write_yaml_config(config=snort_module_config,
                                          path=f"{constants.FILEBEAT.MODULES_CONFIG_DIR}"
                                               f"{constants.FILEBEAT.SNORT_MODULE_CONFIG_FILE}")

    @staticmethod
    def set_filebeat_elasticsearch_module_config() -> None:
        """
        Updates the filebeat elasticsearch module configuration

        :return: None
        """
        logging.info(f"Enabling module with command: "
                     f"{constants.FILEBEAT.ENABLE_MODULE_CMD.format(constants.FILEBEAT.ELASTICSEARCH_MODULE)}")
        output = subprocess.run(constants.FILEBEAT.ENABLE_MODULE_CMD.format(
            constants.FILEBEAT.ELASTICSEARCH_MODULE).split(" "), capture_output=True, text=True)
        logging.info(f"Module enabled, output: {output.stdout}, err output: {output.stderr}")
        elasticsearch_module_config = HostManagerUtil.filebeat_elasticsearch_module_config()
        logging.info(f"Updating filebeat elasticsearch module config: \n{elasticsearch_module_config}")
        HostManagerUtil.write_yaml_config(config=elasticsearch_module_config,
                                          path=f"{constants.FILEBEAT.MODULES_CONFIG_DIR}"
                                               f"{constants.FILEBEAT.ELASTICSEARCH_MODULE_CONFIG_FILE}")

    @staticmethod
    def set_filebeat_logstash_module_config() -> None:
        """
        Updates the filebeat logstash module configuration

        :return: None
        """
        logging.info(f"Enabling module with command: "
                     f"{constants.FILEBEAT.ENABLE_MODULE_CMD.format(constants.FILEBEAT.LOGSTASH_MODULE)}")
        output = subprocess.run(constants.FILEBEAT.ENABLE_MODULE_CMD.format(constants.FILEBEAT.LOGSTASH_MODULE).split(
            " "), capture_output=True, text=True)
        logging.info(f"Module enabled, output: {output.stdout}, err output: {output.stderr}")
        logstash_module_config = HostManagerUtil.filebeat_logstash_module_config()
        logging.info(f"Updating filebeat logstash module config: \n{logstash_module_config}")
        HostManagerUtil.write_yaml_config(config=logstash_module_config,
                                          path=f"{constants.FILEBEAT.MODULES_CONFIG_DIR}"
                                               f"{constants.FILEBEAT.LOGSTASH_MODULE_CONFIG_FILE}")

    @staticmethod
    def set_filebeat_kibana_module_config() -> None:
        """
        Updates the filebeat kibana module configuration

        :return: None
        """
        logging.info(f"Enabling module with command: "
                     f"{constants.FILEBEAT.ENABLE_MODULE_CMD.format(constants.FILEBEAT.KIBANA_MODULE)}")
        output = subprocess.run(constants.FILEBEAT.ENABLE_MODULE_CMD.format(constants.FILEBEAT.KIBANA_MODULE).split(
            " "), capture_output=True, text=True)
        logging.info(f"Module enabled, output: {output.stdout}, err output: {output.stderr}")
        kibana_module_config = HostManagerUtil.filebeat_kibana_module_config()
        logging.info(f"Updating filebeat kibana module config: \n{kibana_module_config}")
        HostManagerUtil.write_yaml_config(config=kibana_module_config,
                                          path=f"{constants.FILEBEAT.MODULES_CONFIG_DIR}"
                                               f"{constants.FILEBEAT.KIBANA_MODULE_CONFIG_FILE}")

    @staticmethod
    def set_filebeat_system_module_config() -> None:
        """
        Updates the filebeat system module configuration

        :return: None
        """
        logging.info(f"Enabling module with command: "
                     f"{constants.FILEBEAT.ENABLE_MODULE_CMD.format(constants.FILEBEAT.SYSTEM_MODULE)}")
        output = subprocess.run(constants.FILEBEAT.ENABLE_MODULE_CMD.format(constants.FILEBEAT.SYSTEM_MODULE).split(
            " "), capture_output=True, text=True)
        logging.info(f"Module enabled, output: {output.stdout}, err output: {output.stderr}")
        system_module_config = HostManagerUtil.filebeat_system_module_config()
        logging.info(f"Updating filebeat system module config: \n{system_module_config}")
        HostManagerUtil.write_yaml_config(config=system_module_config,
                                          path=f"{constants.FILEBEAT.MODULES_CONFIG_DIR}"
                                               f"{constants.FILEBEAT.SYSTEM_MODULE_CONFIG_FILE}")

    @staticmethod
    def set_filebeat_kafka_module_config() -> None:
        """
        Updates the filebeat kafka module configuration

        :return: None
        """
        logging.info(f"Enabling module with command: "
                     f"{constants.FILEBEAT.ENABLE_MODULE_CMD.format(constants.FILEBEAT.KAFKA_MODULE)}")
        output = subprocess.run(constants.FILEBEAT.ENABLE_MODULE_CMD.format(constants.FILEBEAT.KAFKA_MODULE).split(" "),
                                capture_output=True, text=True)
        logging.info(f"Module enabled, output: {output.stdout}, err output: {output.stderr}")
        kafka_module_config = HostManagerUtil.filebeat_kafka_module_config()
        logging.info(f"Updating filebeat kafka module config: \n{kafka_module_config}")
        HostManagerUtil.write_yaml_config(config=kafka_module_config,
                                          path=f"{constants.FILEBEAT.MODULES_CONFIG_DIR}"
                                               f"{constants.FILEBEAT.KAFKA_MODULE_CONFIG_FILE}")

    @staticmethod
    def _get_packetbeat_status() -> bool:
        """
        Utility method to get the status of packetbeat

        :return: status of filebeat
        """
        logging.info(f"Getting packetbeat status with command: {constants.PACKETBEAT.PACKETBEAT_STATUS}")
        output = subprocess.run(constants.PACKETBEAT.PACKETBEAT_STATUS.split(" "), capture_output=True, text=True)
        packetbeat_running = not ("not" in output.stdout)
        logging.info(f"Got packetbeat status, output:{output.stdout}, err output: {output.stderr} ")
        return packetbeat_running

    @staticmethod
    def _start_packetbeat() -> None:
        """
        Utility method to start packetbeat

        :return: None
        """
        logging.info(f"Starting packetbeat with command: {constants.PACKETBEAT.PACKETBEAT_START}")
        output = subprocess.run(constants.PACKETBEAT.PACKETBEAT_START.split(" "), capture_output=True, text=True)
        logging.info(f"Started packetbeat, stdout:{output.stdout}, stderr: {output.stderr}")

    @staticmethod
    def _stop_packetbeat() -> None:
        """
        Utility method to stop packetbeat

        :return: None
        """
        logging.info(f"Stopping packetbeat with command: {constants.PACKETBEAT.PACKETBEAT_STOP}")
        output = subprocess.run(constants.PACKETBEAT.PACKETBEAT_STOP.split(" "), capture_output=True, text=True)
        logging.info(f"Stopped packetbeat, output:{output.stdout}, err output: {output.stderr} ")

    @staticmethod
    def _set_packetbeat_config(kibana_ip: str, kibana_port: int, elastic_ip: str, elastic_port: int,
                               num_elastic_shards: int) -> None:
        """
        Updates the packetbeat configuration file

        :param kibana_ip: the IP of Kibana where the data should be visualized
        :param kibana_port: the port of Kibana where the data should be visualized
        :param elastic_ip: the IP of elastic where the data should be shipped
        :param elastic_port: the port of elastic where the data should be shipped
        :param num_elastic_shards: the number of elastic shards
        :return: None
        """
        packetbeat_config = HostManagerUtil.packetbeat_config(
            kibana_ip=kibana_ip, kibana_port=kibana_port, elastic_ip=elastic_ip, elastic_port=elastic_port,
            num_elastic_shards=num_elastic_shards)

        logging.info(f"Updating packetbeat config: \n{packetbeat_config}")
        HostManagerUtil.write_yaml_config(config=packetbeat_config, path=constants.PACKETBEAT.CONFIG_FILE)
        logging.info(f"Running packetbeat setup command: {constants.PACKETBEAT.SETUP_CMD}")
        output = subprocess.run(constants.PACKETBEAT.SETUP_CMD.split(" "), capture_output=True, text=True)
        logging.info(f"Stdout of the setup command: {output.stdout}, stderr of the setup command: {output.stderr}")

    @staticmethod
    def set_metricbeat_system_module_config() -> None:
        """
        Updates the metricbeat system module configuration

        :return: None
        """
        logging.info(f"Enabling system module with command: "
                     f"{constants.METRICBEAT.ENABLE_MODULE_CMD.format(constants.METRICBEAT.SYSTEM_MODULE)}")
        output = subprocess.run(constants.METRICBEAT.ENABLE_MODULE_CMD.format(
            constants.METRICBEAT.SYSTEM_MODULE).split(" "), capture_output=True, text=True)
        logging.info(f"Module enabled, output: {output.stdout}, err output: {output.stderr}")
        system_module_config = HostManagerUtil.metricbeat_system_module_config()
        logging.info(f"Updating metricbeat system module config: \n{system_module_config}")
        HostManagerUtil.write_yaml_config(config=system_module_config,
                                          path=f"{constants.METRICBEAT.MODULES_CONFIG_DIR}"
                                               f"{constants.METRICBEAT.SYSTEM_MODULE_CONFIG_FILE}")

    @staticmethod
    def set_metricbeat_linux_module_config() -> None:
        """
        Updates the metricbeat system module configuration

        :return: None
        """
        logging.info(f"Enabling linux module with command: "
                     f"{constants.METRICBEAT.ENABLE_MODULE_CMD.format(constants.METRICBEAT.LINUX_MODULE)}")
        output = subprocess.run(constants.METRICBEAT.ENABLE_MODULE_CMD.format(
            constants.METRICBEAT.LINUX_MODULE).split(" "), capture_output=True, text=True)
        logging.info(f"Module enabled, output: {output.stdout}, err output: {output.stderr}")
        linux_module_config = HostManagerUtil.metricbeat_linux_module_config()
        logging.info(f"Updating metricbeat linux module config: \n{linux_module_config}")
        HostManagerUtil.write_yaml_config(config=linux_module_config,
                                          path=f"{constants.METRICBEAT.MODULES_CONFIG_DIR}"
                                               f"{constants.METRICBEAT.LINUX_MODULE_CONFIG_FILE}")

    @staticmethod
    def set_metricbeat_kafka_module_config(kafka_ip: str, kafka_port: int) -> None:
        """
        Updates the metricbeat kafka module configuration

        :param kafka_ip: ip of the kafka server
        :param kafka_port: port of the kafka server
        :return: None
        """
        logging.info(f"Enabling kafka module with command: "
                     f"{constants.METRICBEAT.ENABLE_MODULE_CMD.format(constants.METRICBEAT.KAFKA_MODULE)}")
        output = subprocess.run(constants.METRICBEAT.ENABLE_MODULE_CMD.format(
            constants.METRICBEAT.KAFKA_MODULE).split(" "), capture_output=True, text=True)
        logging.info(f"Module enabled, output: {output.stdout}, err output: {output.stderr}")
        kafka_module_config = HostManagerUtil.metricbeat_kafka_module_config(kafka_ip=kafka_ip, kafka_port=kafka_port)
        logging.info(f"Updating metricbeat kafka module config: \n{kafka_module_config}")
        HostManagerUtil.write_yaml_config(config=kafka_module_config,
                                          path=f"{constants.METRICBEAT.MODULES_CONFIG_DIR}"
                                               f"{constants.METRICBEAT.KAFKA_MODULE_CONFIG_FILE}")

    @staticmethod
    def set_metricbeat_elasticsearch_module_config(elastic_ip: str, elastic_port: int) -> None:
        """
        Updates the metricbeat elastic module configuration

        :param elastic_ip: ip of the elastic server
        :param elastic_port: port of the elastic server
        :return: None
        """
        logging.info(f"Enabling elasticsearch module with command: "
                     f"{constants.METRICBEAT.ENABLE_MODULE_CMD.format(constants.METRICBEAT.ELASTICSEARCH_MODULE)}")
        output = subprocess.run(constants.METRICBEAT.ENABLE_MODULE_CMD.format(
            constants.METRICBEAT.ELASTICSEARCH_MODULE).split(" "), capture_output=True, text=True)
        logging.info(f"Module enabled, output: {output.stdout}, err output: {output.stderr}")
        elasticsearch_module_config = HostManagerUtil.metricbeat_elasticsearch_module_config(elastic_ip=elastic_ip,
                                                                                             elastic_port=elastic_port)
        logging.info(f"Updating metricbeat elasticsearch module config: \n{elasticsearch_module_config}")
        HostManagerUtil.write_yaml_config(config=elasticsearch_module_config,
                                          path=f"{constants.METRICBEAT.MODULES_CONFIG_DIR}"
                                               f"{constants.METRICBEAT.ELASTICSEARCH_MODULE_CONFIG_FILE}")

    @staticmethod
    def set_metricbeat_kibana_module_config(kibana_ip: str, kibana_port: int) -> None:
        """
        Updates the metricbeat kibana module configuration

        :param kibana_ip: ip of the kibana server
        :param kibana_port: port of the kibana server
        :return: None
        """
        logging.info(f"Enabling Kibana module with command: "
                     f"{constants.METRICBEAT.ENABLE_MODULE_CMD.format(constants.METRICBEAT.KIBANA_MODULE)}")
        output = subprocess.run(constants.METRICBEAT.ENABLE_MODULE_CMD.format(
            constants.METRICBEAT.KIBANA_MODULE).split(" "), capture_output=True, text=True)
        logging.info(f"Module enabled, output: {output.stdout}, err output: {output.stderr}")
        kibana_module_config = HostManagerUtil.metricbeat_kibana_module_config(
            kibana_ip=kibana_ip, kibana_port=kibana_port)
        logging.info(f"Updating metricbeat kibana module config: \n{kibana_module_config}")
        HostManagerUtil.write_yaml_config(config=kibana_module_config,
                                          path=f"{constants.METRICBEAT.MODULES_CONFIG_DIR}"
                                               f"{constants.METRICBEAT.KIBANA_MODULE_CONFIG_FILE}")

    @staticmethod
    def set_metricbeat_logstash_module_config(logstash_ip: str, logstash_port: int) -> None:
        """
        Updates the metricbeat logstas module configuration

        :param logstash_ip: ip of the logstash server
        :param logstash_port: port of the logstash server
        :return: None
        """
        logging.info(f"Enabling Logstash module with command: "
                     f"{constants.METRICBEAT.ENABLE_MODULE_CMD.format(constants.METRICBEAT.LOGSTASH_MODULE)}")
        output = subprocess.run(constants.METRICBEAT.ENABLE_MODULE_CMD.format(
            constants.METRICBEAT.LOGSTASH_MODULE).split(" "), capture_output=True, text=True)
        logging.info(f"Module enabled, output: {output.stdout}, err output: {output.stderr}")
        logstash_module_config = HostManagerUtil.metricbeat_logstash_module_config(logstash_ip=logstash_ip,
                                                                                   logstash_port=logstash_port)
        logging.info(f"Updating metricbeat logstash module config: \n{logstash_module_config}")
        HostManagerUtil.write_yaml_config(config=logstash_module_config,
                                          path=f"{constants.METRICBEAT.MODULES_CONFIG_DIR}"
                                               f"{constants.METRICBEAT.LOGSTASH_MODULE_CONFIG_FILE}")

    @staticmethod
    def _get_heartbeat_status() -> bool:
        """
        Utility method to get the status of heartbeat

        :return: status of heartbeat
        """
        logging.info(f"Getting heartbeat status with command: {constants.HEARTBEAT.HEARTBEAT_STATUS}")
        output = subprocess.run(constants.HEARTBEAT.HEARTBEAT_STATUS.split(" "), capture_output=True, text=True)
        heartbeat_running = not ("not" in output.stdout)
        logging.info(f"Got heartbeat status, output:{output.stdout}, err output: {output.stderr} ")
        return heartbeat_running

    @staticmethod
    def _start_heartbeat() -> None:
        """
        Utility method to start heartbeat

        :return: None
        """
        logging.info(f"Starting heartbeat with command: {constants.HEARTBEAT.HEARTBEAT_START}")
        output = subprocess.run(constants.HEARTBEAT.HEARTBEAT_START.split(" "), capture_output=True, text=True)
        logging.info(f"Started heartbeat, stdout:{output.stdout}, stderr: {output.stderr}")

    @staticmethod
    def _stop_heartbeat() -> None:
        """
        Utility method to stop heartbeat

        :return: None
        """
        logging.info(f"Stopping heartbeat with command: {constants.HEARTBEAT.HEARTBEAT_STOP}")
        output = subprocess.run(constants.HEARTBEAT.HEARTBEAT_STOP.split(" "), capture_output=True, text=True)
        logging.info(f"Stopped heartbeat, output:{output.stdout}, err output: {output.stderr} ")

    @staticmethod
    def _read_pid_file(path: str) -> int:
        """
        Reads the PID from a pidfile

        :param path: the path to the file
        :return: the parsed pid, or -1 if the pidfile could not be read
        """
        if os.path.exists(path):
            pid = int(open(path, "r").read())
            return pid
        return -1

    @staticmethod
    def _start_spark() -> None:
        """
        Utility method to start spark

        :return: None
        """
        logging.info(f"Starting spark master with command: {constants.SPARK.START_SPARK_MASTER}")
        output = subprocess.run(constants.SPARK.START_SPARK_MASTER.split(" "), capture_output=True, text=True)
        logging.info(f"Started spark master, stdout:{output.stdout}, stderr: {output.stderr}")
        time.sleep(5)
        logging.info(f"Starting spark worker with command: {constants.SPARK.START_SPARK_WORKER}")
        output = subprocess.run(constants.SPARK.START_SPARK_WORKER.split(" "), capture_output=True, text=True)
        logging.info(f"Started spark worker, stdout:{output.stdout}, stderr: {output.stderr}")

    @staticmethod
    def _stop_spark() -> None:
        """
        Utility method to stop spark

        :return: None
        """
        worker_pid = HostManagerServicer._read_pid_file(constants.SPARK.SPARK_WORKER_PID_FILE)
        logging.info(f"Stopping spark worker with command: {constants.SPARK.STOP_SPARK_WORKER.format(worker_pid)}")
        output = subprocess.run(constants.SPARK.STOP_SPARK_WORKER.format(worker_pid).split(" "),
                                capture_output=True, text=True)
        logging.info(f"Stopped the spark worker, output:{output.stdout}, err output: {output.stderr} ")
        master_pid = HostManagerServicer._read_pid_file(constants.SPARK.SPARK_MASTER_PID_FILE)
        logging.info(f"Stopping spark master with command: {constants.SPARK.STOP_SPARK_MASTER.format(master_pid)}")
        output = subprocess.run(constants.SPARK.STOP_SPARK_MASTER.format(master_pid).split(" "),
                                capture_output=True, text=True)
        logging.info(f"Stopped the spark master, output:{output.stdout}, err output: {output.stderr} ")

    @staticmethod
    def _set_heartbeat_config(kibana_ip: str, kibana_port: int, elastic_ip: str,
                              elastic_port: int, num_elastic_shards: int, hosts_to_monitor: List[str]) -> None:
        """
        Updates the heartbeat configuration file

        :param hosts_to_monitor: the list of hosts to monitor
        :param kibana_ip: the IP of Kibana where the data should be visualized
        :param kibana_port: the port of Kibana where the data should be visualized
        :param elastic_ip: the IP of elastic where the data should be shipped
        :param elastic_port: the port of elastic where the data should be shipped
        :param num_elastic_shards: the number of elastic shards
        :return: None
        """
        heartbeat_config = HostManagerUtil.heartbeat_config(
            kibana_ip=kibana_ip, kibana_port=kibana_port, elastic_ip=elastic_ip, elastic_port=elastic_port,
            num_elastic_shards=num_elastic_shards, hosts_to_monitor=list(hosts_to_monitor))
        logging.info(f"Updating heartbeat config: \n{heartbeat_config}")
        HostManagerUtil.write_yaml_config(config=heartbeat_config, path=constants.HEARTBEAT.CONFIG_FILE)
        logging.info(f"Running heartbeat setup command: {constants.HEARTBEAT.SETUP_CMD}")
        output = subprocess.run(constants.HEARTBEAT.SETUP_CMD.split(" "), capture_output=True, text=True)
        logging.info(f"Stdout of the setup command: {output.stdout}, stderr of the setup command: {output.stderr}")


def serve(port: int = 50049, log_dir: str = "/", max_workers: int = 10,
          log_file_name: str = "host_manager.log") -> None:
    """
    Starts the gRPC server for managing clients

    :param port: the port that the server will listen to
    :param log_dir: the directory to write the log file
    :param log_file_name: the file name of the log
    :param max_workers: the maximum number of GRPC workers
    :return: None
    """
    constants.LOG_FILES.HOST_MANAGER_LOG_DIR = log_dir
    constants.LOG_FILES.HOST_MANAGER_LOG_FILE = log_file_name
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=max_workers))
    csle_collector.host_manager.host_manager_pb2_grpc.add_HostManagerServicer_to_server(
        HostManagerServicer(), server)
    server.add_insecure_port(f'[::]:{port}')
    server.start()
    logging.info(f"HostManager Server Started, Listening on port: {port}")
    server.wait_for_termination()


# Program entrypoint
if __name__ == '__main__':
    serve()



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class GetSnortIdsAlertsMsg(google.protobuf.message.Message):
    """Message that the client sends to extract the Snort IDS log statistics from a given timestamp"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TIMESTAMP_FIELD_NUMBER: builtins.int
    LOG_FILE_PATH_FIELD_NUMBER: builtins.int
    timestamp: builtins.float
    log_file_path: builtins.str
    def __init__(
        self,
        *,
        timestamp: builtins.float = ...,
        log_file_path: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["log_file_path", b"log_file_path", "timestamp", b"timestamp"]) -> None: ...

global___GetSnortIdsAlertsMsg = GetSnortIdsAlertsMsg

@typing_extensions.final
class StopSnortIdsMonitorMsg(google.protobuf.message.Message):
    """Message that the client sends to stop the SnortIdsMonitor"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StopSnortIdsMonitorMsg = StopSnortIdsMonitorMsg

@typing_extensions.final
class StartSnortIdsMonitorMsg(google.protobuf.message.Message):
    """Message that the client sends to start the SnortIdsMonitor"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KAFKA_IP_FIELD_NUMBER: builtins.int
    KAFKA_PORT_FIELD_NUMBER: builtins.int
    LOG_FILE_PATH_FIELD_NUMBER: builtins.int
    TIME_STEP_LEN_SECONDS_FIELD_NUMBER: builtins.int
    kafka_ip: builtins.str
    kafka_port: builtins.int
    log_file_path: builtins.str
    time_step_len_seconds: builtins.int
    def __init__(
        self,
        *,
        kafka_ip: builtins.str = ...,
        kafka_port: builtins.int = ...,
        log_file_path: builtins.str = ...,
        time_step_len_seconds: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["kafka_ip", b"kafka_ip", "kafka_port", b"kafka_port", "log_file_path", b"log_file_path", "time_step_len_seconds", b"time_step_len_seconds"]) -> None: ...

global___StartSnortIdsMonitorMsg = StartSnortIdsMonitorMsg

@typing_extensions.final
class StopSnortIdsMsg(google.protobuf.message.Message):
    """Message that the client sends to stop the Snort IDS"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StopSnortIdsMsg = StopSnortIdsMsg

@typing_extensions.final
class StartSnortIdsMsg(google.protobuf.message.Message):
    """Message that the client sends to start the Snort IDS"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    INGRESS_INTERFACE_FIELD_NUMBER: builtins.int
    EGRESS_INTERFACE_FIELD_NUMBER: builtins.int
    SUBNETMASK_FIELD_NUMBER: builtins.int
    ingress_interface: builtins.str
    egress_interface: builtins.str
    subnetmask: builtins.str
    def __init__(
        self,
        *,
        ingress_interface: builtins.str = ...,
        egress_interface: builtins.str = ...,
        subnetmask: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["egress_interface", b"egress_interface", "ingress_interface", b"ingress_interface", "subnetmask", b"subnetmask"]) -> None: ...

global___StartSnortIdsMsg = StartSnortIdsMsg

@typing_extensions.final
class GetSnortIdsMonitorStatusMsg(google.protobuf.message.Message):
    """Message that the client sends to get the status of the SnortIdsMonitor"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___GetSnortIdsMonitorStatusMsg = GetSnortIdsMonitorStatusMsg

@typing_extensions.final
class SnortIdsMonitorDTO(google.protobuf.message.Message):
    """Message that the server returns when asked about the status of the Snort IDS Monitor thread and Snort itself"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MONITOR_RUNNING_FIELD_NUMBER: builtins.int
    SNORT_IDS_RUNNING_FIELD_NUMBER: builtins.int
    monitor_running: builtins.bool
    snort_ids_running: builtins.bool
    def __init__(
        self,
        *,
        monitor_running: builtins.bool = ...,
        snort_ids_running: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["monitor_running", b"monitor_running", "snort_ids_running", b"snort_ids_running"]) -> None: ...

global___SnortIdsMonitorDTO = SnortIdsMonitorDTO

@typing_extensions.final
class SnortIdsLogDTO(google.protobuf.message.Message):
    """Message that the server returns when requested by the client, contains info about the Snort IDS log"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TIMESTAMP_FIELD_NUMBER: builtins.int
    IP_FIELD_NUMBER: builtins.int
    ATTEMPTED_ADMIN_ALERTS_FIELD_NUMBER: builtins.int
    ATTEMPTED_USER_ALERTS_FIELD_NUMBER: builtins.int
    INAPPROPRIATE_CONTENT_ALERTS_FIELD_NUMBER: builtins.int
    POLICY_VIOLATION_ALERTS_FIELD_NUMBER: builtins.int
    SHELLCODE_DETECT_ALERTS_FIELD_NUMBER: builtins.int
    SUCCESSFUL_ADMIN_ALERTS_FIELD_NUMBER: builtins.int
    SUCCESSFUL_USER_ALERTS_FIELD_NUMBER: builtins.int
    TROJAN_ACTIVITY_ALERTS_FIELD_NUMBER: builtins.int
    UNSUCCESSFUL_USER_ALERTS_FIELD_NUMBER: builtins.int
    WEB_APPLICATION_ATTACK_ALERTS_FIELD_NUMBER: builtins.int
    ATTEMPTED_DOS_ALERTS_FIELD_NUMBER: builtins.int
    ATTEMPTED_RECON_ALERTS_FIELD_NUMBER: builtins.int
    BAD_UNKNOWN_ALERTS_FIELD_NUMBER: builtins.int
    DEFAULT_LOGIN_ATTEMPT_ALERTS_FIELD_NUMBER: builtins.int
    DENIAL_OF_SERVICE_ALERTS_FIELD_NUMBER: builtins.int
    MISC_ATTACK_ALERTS_FIELD_NUMBER: builtins.int
    NON_STANDARD_PROTOCOL_ALERTS_FIELD_NUMBER: builtins.int
    RPC_PORTMAN_DECODE_ALERTS_FIELD_NUMBER: builtins.int
    SUCCESSFUL_DOS_ALERTS_FIELD_NUMBER: builtins.int
    SUCCESSFUL_RECON_LARGESCALE_ALERTS_FIELD_NUMBER: builtins.int
    SUCCESSFUL_RECON_LIMITED_ALERTS_FIELD_NUMBER: builtins.int
    SUSPICIOUS_FILENAME_DETECT_ALERTS_FIELD_NUMBER: builtins.int
    SUSPICIOUS_LOGIN_ALERTS_FIELD_NUMBER: builtins.int
    SYSTEM_CALL_DETECT_ALERTS_FIELD_NUMBER: builtins.int
    UNUSUAL_CLIENT_PORT_CONNECTION_ALERTS_FIELD_NUMBER: builtins.int
    WEB_APPLICATION_ACTIVITY_ALERTS_FIELD_NUMBER: builtins.int
    ICMP_EVENT_ALERTS_FIELD_NUMBER: builtins.int
    MISC_ACTIVITY_ALERTS_FIELD_NUMBER: builtins.int
    NETWORK_SCAN_ALERTS_FIELD_NUMBER: builtins.int
    NOT_SUSPICIOUS_ALERTS_FIELD_NUMBER: builtins.int
    PROTOCOL_COMMAND_DECODE_ALERTS_FIELD_NUMBER: builtins.int
    STRING_DETECT_ALERTS_FIELD_NUMBER: builtins.int
    UNKNOWN_ALERTS_FIELD_NUMBER: builtins.int
    TCP_CONNECTION_ALERTS_FIELD_NUMBER: builtins.int
    PRIORITY_1_ALERTS_FIELD_NUMBER: builtins.int
    PRIORITY_2_ALERTS_FIELD_NUMBER: builtins.int
    PRIORITY_3_ALERTS_FIELD_NUMBER: builtins.int
    PRIORITY_4_ALERTS_FIELD_NUMBER: builtins.int
    TOTAL_ALERTS_FIELD_NUMBER: builtins.int
    WARNING_ALERTS_FIELD_NUMBER: builtins.int
    SEVERE_ALERTS_FIELD_NUMBER: builtins.int
    ALERTS_WEIGHTED_BY_PRIORITY_FIELD_NUMBER: builtins.int
    timestamp: builtins.float
    ip: builtins.str
    attempted_admin_alerts: builtins.int
    attempted_user_alerts: builtins.int
    inappropriate_content_alerts: builtins.int
    policy_violation_alerts: builtins.int
    shellcode_detect_alerts: builtins.int
    successful_admin_alerts: builtins.int
    successful_user_alerts: builtins.int
    trojan_activity_alerts: builtins.int
    unsuccessful_user_alerts: builtins.int
    web_application_attack_alerts: builtins.int
    attempted_dos_alerts: builtins.int
    attempted_recon_alerts: builtins.int
    bad_unknown_alerts: builtins.int
    default_login_attempt_alerts: builtins.int
    denial_of_service_alerts: builtins.int
    misc_attack_alerts: builtins.int
    non_standard_protocol_alerts: builtins.int
    rpc_portman_decode_alerts: builtins.int
    successful_dos_alerts: builtins.int
    successful_recon_largescale_alerts: builtins.int
    successful_recon_limited_alerts: builtins.int
    suspicious_filename_detect_alerts: builtins.int
    suspicious_login_alerts: builtins.int
    system_call_detect_alerts: builtins.int
    unusual_client_port_connection_alerts: builtins.int
    web_application_activity_alerts: builtins.int
    icmp_event_alerts: builtins.int
    misc_activity_alerts: builtins.int
    network_scan_alerts: builtins.int
    not_suspicious_alerts: builtins.int
    protocol_command_decode_alerts: builtins.int
    string_detect_alerts: builtins.int
    unknown_alerts: builtins.int
    tcp_connection_alerts: builtins.int
    priority_1_alerts: builtins.int
    priority_2_alerts: builtins.int
    priority_3_alerts: builtins.int
    priority_4_alerts: builtins.int
    total_alerts: builtins.int
    warning_alerts: builtins.int
    severe_alerts: builtins.int
    alerts_weighted_by_priority: builtins.int
    def __init__(
        self,
        *,
        timestamp: builtins.float = ...,
        ip: builtins.str = ...,
        attempted_admin_alerts: builtins.int = ...,
        attempted_user_alerts: builtins.int = ...,
        inappropriate_content_alerts: builtins.int = ...,
        policy_violation_alerts: builtins.int = ...,
        shellcode_detect_alerts: builtins.int = ...,
        successful_admin_alerts: builtins.int = ...,
        successful_user_alerts: builtins.int = ...,
        trojan_activity_alerts: builtins.int = ...,
        unsuccessful_user_alerts: builtins.int = ...,
        web_application_attack_alerts: builtins.int = ...,
        attempted_dos_alerts: builtins.int = ...,
        attempted_recon_alerts: builtins.int = ...,
        bad_unknown_alerts: builtins.int = ...,
        default_login_attempt_alerts: builtins.int = ...,
        denial_of_service_alerts: builtins.int = ...,
        misc_attack_alerts: builtins.int = ...,
        non_standard_protocol_alerts: builtins.int = ...,
        rpc_portman_decode_alerts: builtins.int = ...,
        successful_dos_alerts: builtins.int = ...,
        successful_recon_largescale_alerts: builtins.int = ...,
        successful_recon_limited_alerts: builtins.int = ...,
        suspicious_filename_detect_alerts: builtins.int = ...,
        suspicious_login_alerts: builtins.int = ...,
        system_call_detect_alerts: builtins.int = ...,
        unusual_client_port_connection_alerts: builtins.int = ...,
        web_application_activity_alerts: builtins.int = ...,
        icmp_event_alerts: builtins.int = ...,
        misc_activity_alerts: builtins.int = ...,
        network_scan_alerts: builtins.int = ...,
        not_suspicious_alerts: builtins.int = ...,
        protocol_command_decode_alerts: builtins.int = ...,
        string_detect_alerts: builtins.int = ...,
        unknown_alerts: builtins.int = ...,
        tcp_connection_alerts: builtins.int = ...,
        priority_1_alerts: builtins.int = ...,
        priority_2_alerts: builtins.int = ...,
        priority_3_alerts: builtins.int = ...,
        priority_4_alerts: builtins.int = ...,
        total_alerts: builtins.int = ...,
        warning_alerts: builtins.int = ...,
        severe_alerts: builtins.int = ...,
        alerts_weighted_by_priority: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["alerts_weighted_by_priority", b"alerts_weighted_by_priority", "attempted_admin_alerts", b"attempted_admin_alerts", "attempted_dos_alerts", b"attempted_dos_alerts", "attempted_recon_alerts", b"attempted_recon_alerts", "attempted_user_alerts", b"attempted_user_alerts", "bad_unknown_alerts", b"bad_unknown_alerts", "default_login_attempt_alerts", b"default_login_attempt_alerts", "denial_of_service_alerts", b"denial_of_service_alerts", "icmp_event_alerts", b"icmp_event_alerts", "inappropriate_content_alerts", b"inappropriate_content_alerts", "ip", b"ip", "misc_activity_alerts", b"misc_activity_alerts", "misc_attack_alerts", b"misc_attack_alerts", "network_scan_alerts", b"network_scan_alerts", "non_standard_protocol_alerts", b"non_standard_protocol_alerts", "not_suspicious_alerts", b"not_suspicious_alerts", "policy_violation_alerts", b"policy_violation_alerts", "priority_1_alerts", b"priority_1_alerts", "priority_2_alerts", b"priority_2_alerts", "priority_3_alerts", b"priority_3_alerts", "priority_4_alerts", b"priority_4_alerts", "protocol_command_decode_alerts", b"protocol_command_decode_alerts", "rpc_portman_decode_alerts", b"rpc_portman_decode_alerts", "severe_alerts", b"severe_alerts", "shellcode_detect_alerts", b"shellcode_detect_alerts", "string_detect_alerts", b"string_detect_alerts", "successful_admin_alerts", b"successful_admin_alerts", "successful_dos_alerts", b"successful_dos_alerts", "successful_recon_largescale_alerts", b"successful_recon_largescale_alerts", "successful_recon_limited_alerts", b"successful_recon_limited_alerts", "successful_user_alerts", b"successful_user_alerts", "suspicious_filename_detect_alerts", b"suspicious_filename_detect_alerts", "suspicious_login_alerts", b"suspicious_login_alerts", "system_call_detect_alerts", b"system_call_detect_alerts", "tcp_connection_alerts", b"tcp_connection_alerts", "timestamp", b"timestamp", "total_alerts", b"total_alerts", "trojan_activity_alerts", b"trojan_activity_alerts", "unknown_alerts", b"unknown_alerts", "unsuccessful_user_alerts", b"unsuccessful_user_alerts", "unusual_client_port_connection_alerts", b"unusual_client_port_connection_alerts", "warning_alerts", b"warning_alerts", "web_application_activity_alerts", b"web_application_activity_alerts", "web_application_attack_alerts", b"web_application_attack_alerts"]) -> None: ...

global___SnortIdsLogDTO = SnortIdsLogDTO
